@@ -1,40 +1,28 @@
 /*
-  SLEPc eigensolver: "davidson"
-
-  Step: calc the best eigenpairs in the subspace V.
-
-  For that, performs these steps:
-    1) Update W <- A * V
-    2) Update H <- V' * W
-    3) Obtain eigenpairs of H
-    4) Select some eigenpairs
-    5) Compute the Ritz pairs of the selected ones
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   SLEPc eigensolver: "davidson"
+
+   Step: calculate the best eigenpairs in the subspace V
+
+   For that, performs these steps:
+     1) Update W <- A * V
+     2) Update H <- V' * W
+     3) Obtain eigenpairs of H
+     4) Select some eigenpairs
+     5) Compute the Ritz pairs of the selected ones
+*/
 
 #include "davidson.h"
 #include <slepcblaslapack.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "dvd_calcpairs_qz_start"
 static PetscErrorCode dvd_calcpairs_qz_start(dvdDashboard *d)
 {
   PetscErrorCode ierr;
@@ -47,8 +35,6 @@ static PetscErrorCode dvd_calcpairs_qz_start(dvdDashboard *d)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "dvd_calcpairs_qz_d"
 static PetscErrorCode dvd_calcpairs_qz_d(dvdDashboard *d)
 {
   PetscErrorCode  ierr;
@@ -59,16 +45,14 @@ static PetscErrorCode dvd_calcpairs_qz_d(dvdDashboard *d)
   ierr = BVDestroy(&d->BX);CHKERRQ(ierr);
   ierr = BVDestroy(&d->auxBV);CHKERRQ(ierr);
   ierr = MatDestroy(&d->H);CHKERRQ(ierr);
-  if (d->G) { ierr = MatDestroy(&d->G);CHKERRQ(ierr); }
+  ierr = MatDestroy(&d->G);CHKERRQ(ierr);
   ierr = MatDestroy(&d->auxM);CHKERRQ(ierr);
   ierr = SlepcVecPoolDestroy(&d->auxV);CHKERRQ(ierr);
   ierr = PetscFree(d->nBds);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-/* in complex, d->size_H real auxiliar values are needed */
-#undef __FUNCT__
-#define __FUNCT__ "dvd_calcpairs_projeig_solve"
+/* in complex, d->size_H real auxiliary values are needed */
 static PetscErrorCode dvd_calcpairs_projeig_solve(dvdDashboard *d)
 {
   PetscErrorCode    ierr;
@@ -105,8 +89,6 @@ static PetscErrorCode dvd_calcpairs_projeig_solve(dvdDashboard *d)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSXDUpdateProj"
 /*
    A(lA:kA-1,lA:kA-1) <- Z(l:k-1)'*A(l:k-1,l:k-1)*Q(l,k-1), where k=l+kA-lA
  */
@@ -156,16 +138,14 @@ static PetscErrorCode EPSXDUpdateProj(Mat Q,Mat Z,PetscInt l,Mat A,PetscInt lA,P
   PetscStackCallBLAS("BLASgemm",BLASgemm_("N","N",&nQ,&dA,&nQ,&one,&pA[ldA*lA+lA],&ldA,&pQ[ldQ*l+l],&ldQ,&zero,pW,&nQ));
   /* A = Q'*W */
   PetscStackCallBLAS("BLASgemm",BLASgemm_("C","N",&dA,&dA,&nQ,&one,&pZ[ldZ*l+l],&ldZ,pW,&nQ,&zero,&pA[ldA*lA+lA],&ldA));
-  ierr = MatDenseGetArray(A,&pA);CHKERRQ(ierr);
-  ierr = MatDenseGetArray(Q,&pQ);CHKERRQ(ierr);
+  ierr = MatDenseRestoreArray(A,&pA);CHKERRQ(ierr);
+  ierr = MatDenseRestoreArray(Q,&pQ);CHKERRQ(ierr);
   if (Q!=Z) { ierr = MatDenseGetArray(Z,&pZ);CHKERRQ(ierr); }
   else pZ = pQ;
-  ierr = MatDenseGetArray(aux,&pW);CHKERRQ(ierr);
+  ierr = MatDenseRestoreArray(aux,&pW);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "dvd_calcpairs_updateproj"
 static PetscErrorCode dvd_calcpairs_updateproj(dvdDashboard *d)
 {
   PetscErrorCode ierr;
@@ -180,10 +160,10 @@ static PetscErrorCode dvd_calcpairs_updateproj(dvdDashboard *d)
   ierr = BVGetActiveColumns(d->eps->V,&lV,&kV);CHKERRQ(ierr);
   ierr = EPSXDUpdateProj(Q,Z,0,d->H,lV,lV+d->V_tra_e,d->auxM);CHKERRQ(ierr);
   if (d->G) { ierr = EPSXDUpdateProj(Q,Z,0,d->G,lV,lV+d->V_tra_e,d->auxM);CHKERRQ(ierr); }
-  ierr = DSRestoreMat(d->eps->ds,DS_MAT_Q,&Q);CHKERRQ(ierr);
-  if (d->W) { ierr = DSRestoreMat(d->eps->ds,DS_MAT_Z,&Z);CHKERRQ(ierr); }
+  ierr = MatDestroy(&Q);CHKERRQ(ierr);
+  if (d->W) { ierr = MatDestroy(&Z);CHKERRQ(ierr); }
 
-  ierr = PetscObjectTypeCompareAny((PetscObject)d->eps->ds,&symm,DSHEP,DSGHIEP,"");CHKERRQ(ierr);
+  ierr = PetscObjectTypeCompareAny((PetscObject)d->eps->ds,&symm,DSHEP,DSGHIEP,DSGHEP,"");CHKERRQ(ierr);
   if (d->V_tra_s==0 || symm) PetscFunctionReturn(0);
   /* Compute upper part of H (and G): H(0:l-1,l:k-1) <- W(0:l-1)' * AV(l:k-1), where
      k=l+d->V_tra_s */
@@ -212,8 +192,6 @@ static PetscErrorCode dvd_calcpairs_updateproj(dvdDashboard *d)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "dvd_calcpairs_updateBV0_gen"
 /*
    BV <- BV*MT
  */
@@ -228,15 +206,13 @@ PETSC_STATIC_INLINE PetscErrorCode dvd_calcpairs_updateBV0_gen(dvdDashboard *d,B
   ierr = MatCreateSeqDense(PETSC_COMM_SELF,k,k,NULL,&auxM);CHKERRQ(ierr);
   ierr = MatZeroEntries(auxM);CHKERRQ(ierr);
   ierr = DSGetDimensions(d->eps->ds,&n,NULL,NULL,NULL,NULL);CHKERRQ(ierr);
-  if (k-l!=n) SETERRQ(PETSC_COMM_SELF,1, "Consistency broken");
+  if (k-l!=n) SETERRQ(PETSC_COMM_SELF,1,"Consistency broken");
   ierr = DSCopyMat(d->eps->ds,mat,0,0,auxM,l,l,n,d->V_tra_e,PETSC_TRUE);CHKERRQ(ierr);
   ierr = BVMultInPlace(bv,auxM,l,l+d->V_tra_e);CHKERRQ(ierr);
   ierr = MatDestroy(&auxM);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "dvd_calcpairs_proj"
 static PetscErrorCode dvd_calcpairs_proj(dvdDashboard *d)
 {
   PetscErrorCode ierr;
@@ -273,7 +249,7 @@ static PetscErrorCode dvd_calcpairs_proj(dvdDashboard *d)
     ierr = dvd_orthV(d->eps->V,l+d->V_new_s,l+d->V_new_e);CHKERRQ(ierr);
     /* 3. AV <- [AV A * V(V_new_s:V_new_e-1)] */
     /* Check consistency */
-    if (k-l != d->V_new_s) SETERRQ(PETSC_COMM_SELF,1, "Consistency broken");
+    if (k-l != d->V_new_s) SETERRQ(PETSC_COMM_SELF,1,"Consistency broken");
     for (i=l+d->V_new_s;i<l+d->V_new_e;i++) {
       ierr = BVGetColumn(d->eps->V,i,&v1);CHKERRQ(ierr);
       ierr = BVGetColumn(d->AX,i,&v2);CHKERRQ(ierr);
@@ -284,7 +260,7 @@ static PetscErrorCode dvd_calcpairs_proj(dvdDashboard *d)
     /* 4. BV <- [BV B * V(V_new_s:V_new_e-1)] */
     if (d->BX) {
       /* Check consistency */
-      if (k-l != d->V_new_s) SETERRQ(PETSC_COMM_SELF,1, "Consistency broken");
+      if (k-l != d->V_new_s) SETERRQ(PETSC_COMM_SELF,1,"Consistency broken");
       for (i=l+d->V_new_s;i<l+d->V_new_e;i++) {
         ierr = BVGetColumn(d->eps->V,i,&v1);CHKERRQ(ierr);
         ierr = BVGetColumn(d->BX,i,&v2);CHKERRQ(ierr);
@@ -329,8 +305,6 @@ static PetscErrorCode dvd_calcpairs_proj(dvdDashboard *d)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "dvd_calcpairs_apply_arbitrary"
 static PetscErrorCode dvd_calcpairs_apply_arbitrary(dvdDashboard *d,PetscInt r_s,PetscInt r_e,PetscScalar *rr,PetscScalar *ri)
 {
   PetscInt       i,k,ld;
@@ -394,8 +368,6 @@ static PetscErrorCode dvd_calcpairs_apply_arbitrary(dvdDashboard *d,PetscInt r_s
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "dvd_calcpairs_selectPairs"
 static PetscErrorCode dvd_calcpairs_selectPairs(dvdDashboard *d,PetscInt n)
 {
   PetscInt       k,lV,kV,nV;
@@ -404,7 +376,7 @@ static PetscErrorCode dvd_calcpairs_selectPairs(dvdDashboard *d,PetscInt n)
 
   PetscFunctionBegin;
   ierr = BVGetActiveColumns(d->eps->V,&lV,&kV);CHKERRQ(ierr);
-  nV = kV - lV; 
+  nV = kV - lV;
   n = PetscMin(n,nV);
   if (n <= 0) PetscFunctionReturn(0);
   /* Put the best n pairs at the beginning. Useful for restarting */
@@ -429,6 +401,8 @@ static PetscErrorCode dvd_calcpairs_selectPairs(dvdDashboard *d,PetscInt n)
     k = 1;
     ierr = DSSort(d->eps->ds,d->eigr,d->eigi,rr,ri,&k);CHKERRQ(ierr);
   }
+  ierr = DSSynchronize(d->eps->ds,d->eigr,d->eigi);CHKERRQ(ierr);
+
   if (d->calcpairs_eigs_trans) {
     ierr = d->calcpairs_eigs_trans(d);CHKERRQ(ierr);
   }
@@ -439,8 +413,6 @@ static PetscErrorCode dvd_calcpairs_selectPairs(dvdDashboard *d,PetscInt n)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSXDComputeDSConv"
 static PetscErrorCode EPSXDComputeDSConv(dvdDashboard *d)
 {
   PetscErrorCode    ierr;
@@ -474,6 +446,7 @@ static PetscErrorCode EPSXDComputeDSConv(dvdDashboard *d)
   }
   ierr = DSSetState(d->eps->ds,DS_STATE_RAW);CHKERRQ(ierr);
   ierr = DSSolve(d->eps->ds,d->eps->eigr,d->eps->eigi);CHKERRQ(ierr);
+  ierr = DSSynchronize(d->eps->ds,d->eps->eigr,d->eps->eigi);CHKERRQ(ierr);
   if (d->W) {
     for (i=0; i<d->eps->nconv; i++) {
       ierr = d->calcpairs_eig_backtrans(d,d->eps->eigr[i],d->eps->eigi[i],&d->eps->eigr[i],&d->eps->eigi[i]);CHKERRQ(ierr);
@@ -482,8 +455,6 @@ static PetscErrorCode EPSXDComputeDSConv(dvdDashboard *d)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "dvd_calcpairs_res_0"
 /*
    Compute the residual vectors R(i) <- (AV - BV*eigr(i))*pX(i), and also
    the norm associated to the Schur pair, where i = r_s..r_e
@@ -514,8 +485,6 @@ static PetscErrorCode dvd_calcpairs_res_0(dvdDashboard *d,PetscInt r_s,PetscInt
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "dvd_calcpairs_proj_res"
 static PetscErrorCode dvd_calcpairs_proj_res(dvdDashboard *d,PetscInt r_s,PetscInt r_e,Vec *R)
 {
   PetscInt       i,l,k;
@@ -549,12 +518,10 @@ static PetscErrorCode dvd_calcpairs_proj_res(dvdDashboard *d,PetscInt r_s,PetscI
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "dvd_calcpairs_qz"
-PetscErrorCode dvd_calcpairs_qz(dvdDashboard *d,dvdBlackboard *b,PetscBool borth,PetscInt cX_proj,PetscBool harm)
+PetscErrorCode dvd_calcpairs_qz(dvdDashboard *d,dvdBlackboard *b,PetscBool borth,PetscBool harm)
 {
   PetscErrorCode ierr;
-  PetscBool      std_probl,her_probl,ind_probl,her_ind_probl;
+  PetscBool      std_probl,her_probl,ind_probl;
   DSType         dstype;
   Vec            v1;
 
@@ -562,17 +529,13 @@ PetscErrorCode dvd_calcpairs_qz(dvdDashboard *d,dvdBlackboard *b,PetscBool borth
   std_probl = DVD_IS(d->sEP,DVD_EP_STD)? PETSC_TRUE: PETSC_FALSE;
   her_probl = DVD_IS(d->sEP,DVD_EP_HERMITIAN)? PETSC_TRUE: PETSC_FALSE;
   ind_probl = DVD_IS(d->sEP,DVD_EP_INDEFINITE)? PETSC_TRUE: PETSC_FALSE;
-  her_ind_probl = (her_probl||ind_probl)? PETSC_TRUE: PETSC_FALSE;
 
   /* Setting configuration constrains */
-  b->max_size_proj = PetscMax(b->max_size_proj,b->max_size_V+cX_proj);
+  b->max_size_proj = PetscMax(b->max_size_proj,b->max_size_V);
   d->W_shift = d->B? PETSC_TRUE: PETSC_FALSE;
-  if (d->B && her_ind_probl && !borth) d->BV_shift = PETSC_TRUE;
-  else d->BV_shift = PETSC_FALSE;
 
   /* Setup the step */
   if (b->state >= DVD_STATE_CONF) {
-    d->max_cX_in_proj = cX_proj;
     d->max_size_P = b->max_size_P;
     d->max_size_proj = b->max_size_proj;
     /* Create a DS if the method works with Schur decompositions */
@@ -602,7 +565,7 @@ PetscErrorCode dvd_calcpairs_qz(dvdDashboard *d,dvdBlackboard *b,PetscBool borth
       ierr = BVDuplicateResize(d->eps->V,d->eps->ncv,&d->BX);CHKERRQ(ierr);
       ierr = BVSetMatrix(d->BX,NULL,PETSC_FALSE);CHKERRQ(ierr);
     } else d->BX = NULL;
-    ierr = MatCreateVecs(d->A,&v1,NULL);CHKERRQ(ierr);
+    ierr = MatCreateVecsEmpty(d->A,&v1,NULL);CHKERRQ(ierr);
     ierr = SlepcVecPoolCreate(v1,0,&d->auxV);CHKERRQ(ierr);
     ierr = VecDestroy(&v1);CHKERRQ(ierr);
     /* Create projected problem matrices */
