@@ -1,27 +1,17 @@
 /*
-  SLEPc eigensolver: "davidson"
-
-  Step: improve the eigenvectors X
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   SLEPc eigensolver: "davidson"
+
+   Step: improve the eigenvectors X
+*/
 
 #include "davidson.h"
 #include <slepcblaslapack.h>
@@ -54,8 +44,6 @@ typedef struct {
   PetscBLASInt *iXKZPivots;        /* array of pivots */
 } dvdImprovex_jd;
 
-#undef __FUNCT__
-#define __FUNCT__ "dvd_improvex_apply_proj"
 /*
    Compute (I - KZ*iXKZ*X')*V where,
    V, the vectors to apply the projector,
@@ -65,7 +53,7 @@ static PetscErrorCode dvd_improvex_apply_proj(dvdDashboard *d,Vec *V,PetscInt cV
 {
 #if defined(PETSC_MISSING_LAPACK_GETRS)
   PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"GETRS - Lapack routines are unavailable");
+  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"GETRS - Lapack routine is unavailable");
 #else
   PetscErrorCode ierr;
   dvdImprovex_jd *data = (dvdImprovex_jd*)d->improveX_data;
@@ -101,7 +89,7 @@ static PetscErrorCode dvd_improvex_apply_proj(dvdDashboard *d,Vec *V,PetscInt cV
   ierr = PetscFPTrapPush(PETSC_FP_TRAP_OFF);CHKERRQ(ierr);
   PetscStackCallBLAS("LAPACKgetrs",LAPACKgetrs_("N",&n,&cV_,data->iXKZ,&ld,data->iXKZPivots,h,&n,&info));
   ierr = PetscFPTrapPop();CHKERRQ(ierr);
-  if (info) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"Error in Lapack XGETRS %d",info);
+  SlepcCheckLapackInfo("getrs",info);
 
   /* V <- V - KZ*h */
   ierr = BVSetActiveColumns(data->KZ,0,k);CHKERRQ(ierr);
@@ -114,8 +102,6 @@ static PetscErrorCode dvd_improvex_apply_proj(dvdDashboard *d,Vec *V,PetscInt cV
 #endif
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "dvd_improvex_applytrans_proj"
 /*
    Compute (I - X*iXKZ*KZ')*V where,
    V, the vectors to apply the projector,
@@ -125,7 +111,7 @@ static PetscErrorCode dvd_improvex_applytrans_proj(dvdDashboard *d,Vec *V,PetscI
 {
 #if defined(PETSC_MISSING_LAPACK_GETRS)
   PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"GETRS - Lapack routines are unavailable");
+  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"GETRS - Lapack routine is unavailable");
   PetscFunctionReturn(0);
 #else
   PetscErrorCode ierr;
@@ -162,7 +148,7 @@ static PetscErrorCode dvd_improvex_applytrans_proj(dvdDashboard *d,Vec *V,PetscI
   ierr = PetscFPTrapPush(PETSC_FP_TRAP_OFF);CHKERRQ(ierr);
   PetscStackCallBLAS("LAPACKgetrs",LAPACKgetrs_("C",&n,&cV_,data->iXKZ,&ld,data->iXKZPivots,h,&n,&info));
   ierr = PetscFPTrapPop();CHKERRQ(ierr);
-  if (info) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"Error in Lapack XGETRS %d",info);
+  SlepcCheckLapackInfo("getrs",info);
 
   /* V <- V - U*h */
   ierr = BVSetActiveColumns(data->U,0,k);CHKERRQ(ierr);
@@ -175,15 +161,13 @@ static PetscErrorCode dvd_improvex_applytrans_proj(dvdDashboard *d,Vec *V,PetscI
 #endif
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "dvd_improvex_jd_end"
 static PetscErrorCode dvd_improvex_jd_end(dvdDashboard *d)
 {
   PetscErrorCode ierr;
   dvdImprovex_jd *data = (dvdImprovex_jd*)d->improveX_data;
 
   PetscFunctionBegin;
-  if (data->friends) { ierr = VecDestroy(&data->friends);CHKERRQ(ierr); }
+  ierr = VecDestroy(&data->friends);CHKERRQ(ierr);
 
   /* Restore the pc of ksp */
   if (data->old_pc) {
@@ -193,8 +177,6 @@ static PetscErrorCode dvd_improvex_jd_end(dvdDashboard *d)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "dvd_improvex_jd_d"
 static PetscErrorCode dvd_improvex_jd_d(dvdDashboard *d)
 {
   PetscErrorCode ierr;
@@ -211,8 +193,6 @@ static PetscErrorCode dvd_improvex_jd_d(dvdDashboard *d)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "dvd_aux_matmult"
 /*
    y <- theta[1]A*x - theta[0]*B*x
    auxV, two auxiliary vectors
@@ -259,8 +239,6 @@ PETSC_STATIC_INLINE PetscErrorCode dvd_aux_matmult(dvdImprovex_jd *data,const Ve
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "dvd_aux_matmulttrans"
 /*
    y <- theta[1]'*A'*x - theta[0]'*B'*x
  */
@@ -306,8 +284,6 @@ PETSC_STATIC_INLINE PetscErrorCode dvd_aux_matmulttrans(dvdImprovex_jd *data,con
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCApplyBA_dvd"
 static PetscErrorCode PCApplyBA_dvd(PC pc,PCSide side,Vec in,Vec out,Vec w)
 {
   PetscErrorCode ierr;
@@ -367,8 +343,6 @@ static PetscErrorCode PCApplyBA_dvd(PC pc,PCSide side,Vec in,Vec out,Vec w)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCApply_dvd"
 static PetscErrorCode PCApply_dvd(PC pc,Vec in,Vec out)
 {
   PetscErrorCode ierr;
@@ -392,8 +366,6 @@ static PetscErrorCode PCApply_dvd(PC pc,Vec in,Vec out)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCApplyTranspose_dvd"
 static PetscErrorCode PCApplyTranspose_dvd(PC pc,Vec in,Vec out)
 {
   PetscErrorCode ierr;
@@ -424,8 +396,6 @@ static PetscErrorCode PCApplyTranspose_dvd(PC pc,Vec in,Vec out)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMult_dvd_jd"
 static PetscErrorCode MatMult_dvd_jd(Mat A,Vec in,Vec out)
 {
   PetscErrorCode ierr;
@@ -449,8 +419,6 @@ static PetscErrorCode MatMult_dvd_jd(Mat A,Vec in,Vec out)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMultTranspose_dvd_jd"
 static PetscErrorCode MatMultTranspose_dvd_jd(Mat A,Vec in,Vec out)
 {
   PetscErrorCode ierr;
@@ -484,8 +452,6 @@ static PetscErrorCode MatMultTranspose_dvd_jd(Mat A,Vec in,Vec out)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatCreateVecs_dvd_jd"
 static PetscErrorCode MatCreateVecs_dvd_jd(Mat A,Vec *right,Vec *left)
 {
   PetscErrorCode ierr;
@@ -527,8 +493,6 @@ static PetscErrorCode MatCreateVecs_dvd_jd(Mat A,Vec *right,Vec *left)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "dvd_improvex_jd_start"
 static PetscErrorCode dvd_improvex_jd_start(dvdDashboard *d)
 {
   PetscErrorCode ierr;
@@ -576,7 +540,7 @@ static PetscErrorCode dvd_improvex_jd_start(dvdDashboard *d)
     ierr = MatCreateShell(PetscObjectComm((PetscObject)d->A),rlA*data->ksp_max_size,clA*data->ksp_max_size,rA*data->ksp_max_size,cA*data->ksp_max_size,data,&A);CHKERRQ(ierr);
     ierr = MatShellSetOperation(A,MATOP_MULT,(void(*)(void))MatMult_dvd_jd);CHKERRQ(ierr);
     ierr = MatShellSetOperation(A,MATOP_MULT_TRANSPOSE,(void(*)(void))MatMultTranspose_dvd_jd);CHKERRQ(ierr);
-    ierr = MatShellSetOperation(A,MATOP_GET_VECS,(void(*)(void))MatCreateVecs_dvd_jd);CHKERRQ(ierr);
+    ierr = MatShellSetOperation(A,MATOP_CREATE_VECS,(void(*)(void))MatCreateVecs_dvd_jd);CHKERRQ(ierr);
 
     /* Try to avoid KSPReset */
     ierr = KSPGetOperatorsSet(data->ksp,&t,NULL);CHKERRQ(ierr);
@@ -600,8 +564,6 @@ static PetscErrorCode dvd_improvex_jd_start(dvdDashboard *d)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "dvd_improvex_jd_proj_cuv"
 /*
   Compute: u <- X, v <- K*(theta[0]*A+theta[1]*B)*X,
   kr <- K^{-1}*(A-eig*B)*X, being X <- V*pX[i_s..i_e-1], Y <- W*pY[i_s..i_e-1]
@@ -627,13 +589,13 @@ static PetscErrorCode dvd_improvex_jd_proj_cuv(dvdDashboard *d,PetscInt i_s,Pets
   /* Check consistency */
   ierr = BVGetActiveColumns(d->eps->V,&lv,&kv);CHKERRQ(ierr);
   V_new = lv - data->size_cX;
-  if (V_new > data->old_size_X) SETERRQ(PETSC_COMM_SELF,1, "Consistency broken");
+  if (V_new > data->old_size_X) SETERRQ(PETSC_COMM_SELF,1,"Consistency broken");
   data->old_size_X = n;
   data->size_cX = lv;
 
   /* KZ <- KZ(rm:rm+max_cX-1) */
   ierr = BVGetActiveColumns(data->KZ,&lKZ,&kKZ);CHKERRQ(ierr);
-  rm = PetscMax(V_new+lKZ-d->max_cX_in_impr,0);
+  rm = PetscMax(V_new+lKZ,0);
   if (rm > 0) {
     for (i=0;i<lKZ;i++) {
       ierr = BVCopyColumn(data->KZ,i+rm,i);CHKERRQ(ierr);
@@ -647,7 +609,7 @@ static PetscErrorCode dvd_improvex_jd_proj_cuv(dvdDashboard *d,PetscInt i_s,Pets
       ierr = PetscMemcpy(&data->XKZ[i*data->ldXKZ+i],&data->XKZ[(i+rm)*data->ldXKZ+i+rm],sizeof(PetscScalar)*lKZ);CHKERRQ(ierr);
     }
   }
-  lKZ = PetscMin(d->max_cX_in_impr,lKZ+V_new);
+  lKZ = PetscMin(0,lKZ+V_new);
   ierr = BVSetActiveColumns(data->KZ,lKZ,lKZ+n);CHKERRQ(ierr);
   ierr = BVSetActiveColumns(data->U,lKZ,lKZ+n);CHKERRQ(ierr);
 
@@ -686,13 +648,11 @@ static PetscErrorCode dvd_improvex_jd_proj_cuv(dvdDashboard *d,PetscInt i_s,Pets
   ierr = PetscFPTrapPush(PETSC_FP_TRAP_OFF);CHKERRQ(ierr);
   PetscStackCallBLAS("LAPACKgetrf",LAPACKgetrf_(&s,&s,data->iXKZ,&ldXKZ,data->iXKZPivots,&info));
   ierr = PetscFPTrapPop();CHKERRQ(ierr);
-  if (info) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"Error in Lapack XGETRF %d",info);
+  SlepcCheckLapackInfo("getrf",info);
   PetscFunctionReturn(0);
 #endif
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "dvd_improvex_jd_gen"
 static PetscErrorCode dvd_improvex_jd_gen(dvdDashboard *d,PetscInt r_s,PetscInt r_e,PetscInt *size_D)
 {
   dvdImprovex_jd *data = (dvdImprovex_jd*)d->improveX_data;
@@ -725,7 +685,7 @@ static PetscErrorCode dvd_improvex_jd_gen(dvdDashboard *d,PetscInt r_s,PetscInt
   for (i=0,s=0;i<n;i+=s) {
     /* If the selected eigenvalue is complex, but the arithmetic is real... */
 #if !defined(PETSC_USE_COMPLEX)
-    if (d->eigi[i] != 0.0) {
+    if (d->eigi[r_s+i] != 0.0) {
       if (i+2 <= max_size_D) s=2;
       else break;
     } else
@@ -760,8 +720,9 @@ static PetscErrorCode dvd_improvex_jd_gen(dvdDashboard *d,PetscInt r_s,PetscInt
 
     /* Check if the first eigenpairs are converged */
     if (i == 0) {
-      ierr = d->preTestConv(d,0,s,s,&d->npreconv);CHKERRQ(ierr);
-      if (d->npreconv > 0) break;
+      PetscInt oldnpreconv = d->npreconv;
+      ierr = d->preTestConv(d,0,r_s+s,r_s+s,&d->npreconv);CHKERRQ(ierr);
+      if (d->npreconv > oldnpreconv) break;
     }
 
     /* Test the odd situation of solving Ax=b with A=I */
@@ -781,8 +742,8 @@ static PetscErrorCode dvd_improvex_jd_gen(dvdDashboard *d,PetscInt r_s,PetscInt
       kr0[0] = kr[0];
       kr0[1] = (s==2 ? kr[1] : NULL);
       ierr = VecCreateCompWithVecs(kr0,data->ksp_max_size,data->friends,&kr_comp);CHKERRQ(ierr);
-      ierr = BVGetColumn(d->eps->V,kV+r_s+i,&D[0]);CHKERRQ(ierr);
-      if (s==2) { ierr = BVGetColumn(d->eps->V,kV+r_s+i+1,&D[1]);CHKERRQ(ierr); }
+      ierr = BVGetColumn(d->eps->V,kV+i,&D[0]);CHKERRQ(ierr);
+      if (s==2) { ierr = BVGetColumn(d->eps->V,kV+i+1,&D[1]);CHKERRQ(ierr); }
       else D[1] = NULL;
       ierr = VecCreateCompWithVecs(D,data->ksp_max_size,data->friends,&D_comp);CHKERRQ(ierr);
       ierr = VecCompSetSubVecs(data->friends,s,NULL);CHKERRQ(ierr);
@@ -795,24 +756,24 @@ static PetscErrorCode dvd_improvex_jd_gen(dvdDashboard *d,PetscInt r_s,PetscInt
       /* Destroy the composed ks and D */
       ierr = VecDestroy(&kr_comp);CHKERRQ(ierr);
       ierr = VecDestroy(&D_comp);CHKERRQ(ierr);
-      ierr = BVRestoreColumn(d->eps->V,kV+r_s+i,&D[0]);CHKERRQ(ierr);
-      if (s==2) { ierr = BVRestoreColumn(d->eps->V,kV+r_s+i+1,&D[1]);CHKERRQ(ierr); }
+      ierr = BVRestoreColumn(d->eps->V,kV+i,&D[0]);CHKERRQ(ierr);
+      if (s==2) { ierr = BVRestoreColumn(d->eps->V,kV+i+1,&D[1]);CHKERRQ(ierr); }
 
     /* If GD */
     } else {
-      ierr = BVGetColumn(d->eps->V,kV+r_s+i,&D[0]);CHKERRQ(ierr);
-      if (s==2) { ierr = BVGetColumn(d->eps->V,kV+r_s+i+1,&D[1]);CHKERRQ(ierr); }
+      ierr = BVGetColumn(d->eps->V,kV+i,&D[0]);CHKERRQ(ierr);
+      if (s==2) { ierr = BVGetColumn(d->eps->V,kV+i+1,&D[1]);CHKERRQ(ierr); }
       for (j=0;j<s;j++) {
         ierr = d->improvex_precond(d,r_s+i+j,kr[j],D[j]);CHKERRQ(ierr);
       }
       ierr = dvd_improvex_apply_proj(d,D,s);CHKERRQ(ierr);
-      ierr = BVRestoreColumn(d->eps->V,kV+r_s+i,&D[0]);CHKERRQ(ierr);
-      if (s==2) { ierr = BVRestoreColumn(d->eps->V,kV+r_s+i+1,&D[1]);CHKERRQ(ierr); }
+      ierr = BVRestoreColumn(d->eps->V,kV+i,&D[0]);CHKERRQ(ierr);
+      if (s==2) { ierr = BVRestoreColumn(d->eps->V,kV+i+1,&D[1]);CHKERRQ(ierr); }
     }
     /* Prevent that short vectors are discarded in the orthogonalization */
     if (i == 0 && d->eps->errest[d->nconv+r_s] > PETSC_MACHINE_EPSILON && d->eps->errest[d->nconv+r_s] < PETSC_MAX_REAL) {
       for (j=0;j<s;j++) {
-        ierr = BVScaleColumn(d->eps->V,kV+r_s+i+j,1.0/d->eps->errest[d->nconv+r_s]);CHKERRQ(ierr);
+        ierr = BVScaleColumn(d->eps->V,kV+i+j,1.0/d->eps->errest[d->nconv+r_s]);CHKERRQ(ierr);
       }
     }
     ierr = SlepcVecPoolRestoreVecs(d->auxV,s,&kr);CHKERRQ(ierr);
@@ -822,9 +783,7 @@ static PetscErrorCode dvd_improvex_jd_gen(dvdDashboard *d,PetscInt r_s,PetscInt
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "dvd_improvex_jd"
-PetscErrorCode dvd_improvex_jd(dvdDashboard *d,dvdBlackboard *b,KSP ksp,PetscInt max_bs,PetscInt cX_impr,PetscBool dynamic)
+PetscErrorCode dvd_improvex_jd(dvdDashboard *d,dvdBlackboard *b,KSP ksp,PetscInt max_bs,PetscBool dynamic)
 {
   PetscErrorCode ierr;
   dvdImprovex_jd *data;
@@ -848,7 +807,7 @@ PetscErrorCode dvd_improvex_jd(dvdDashboard *d,dvdBlackboard *b,KSP ksp,PetscInt
     b->max_size_P = PetscMax(b->max_size_P,1);
   }
   b->max_size_X = PetscMax(b->max_size_X,max_bs);
-  size_P = b->max_size_P+cX_impr;
+  size_P = b->max_size_P;
 
   /* Setup the preconditioner */
   if (ksp) {
@@ -862,7 +821,6 @@ PetscErrorCode dvd_improvex_jd(dvdDashboard *d,dvdBlackboard *b,KSP ksp,PetscInt
   if (b->state >= DVD_STATE_CONF) {
     ierr = PetscNewLog(d->eps,&data);CHKERRQ(ierr);
     data->dynamic = dynamic;
-    d->max_cX_in_impr = cX_impr;
     ierr = PetscMalloc1(size_P*size_P,&data->XKZ);CHKERRQ(ierr);
     ierr = PetscMalloc1(size_P*size_P,&data->iXKZ);CHKERRQ(ierr);
     ierr = PetscMalloc1(size_P,&data->iXKZPivots);CHKERRQ(ierr);
@@ -890,10 +848,8 @@ PetscErrorCode dvd_improvex_jd(dvdDashboard *d,dvdBlackboard *b,KSP ksp,PetscInt
 }
 
 #if !defined(PETSC_USE_COMPLEX)
-#undef __FUNCT__
-#define __FUNCT__ "dvd_complex_rayleigh_quotient"
 PETSC_STATIC_INLINE PetscErrorCode dvd_complex_rayleigh_quotient(Vec ur,Vec ui,Vec Axr,Vec Axi,Vec Bxr,Vec Bxi,PetscScalar *eigr,PetscScalar *eigi)
-{ 
+{
   PetscErrorCode ierr;
   PetscScalar    rAr,iAr,rAi,iAi,rBr,iBr,rBi,iBi,b0,b2,b4,b6,b7;
 
@@ -928,8 +884,6 @@ PETSC_STATIC_INLINE PetscErrorCode dvd_complex_rayleigh_quotient(Vec ur,Vec ui,V
 }
 #endif
 
-#undef __FUNCT__
-#define __FUNCT__ "dvd_compute_n_rr"
 PETSC_STATIC_INLINE PetscErrorCode dvd_compute_n_rr(PetscInt i_s,PetscInt n,PetscScalar *eigr,PetscScalar *eigi,Vec *u,Vec *Ax,Vec *Bx)
 {
   PetscErrorCode ierr;
@@ -962,8 +916,6 @@ PETSC_STATIC_INLINE PetscErrorCode dvd_compute_n_rr(PetscInt i_s,PetscInt n,Pets
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "dvd_improvex_jd_proj_uv_KZX"
 /*
   Compute: u <- X, v <- K*(theta[0]*A+theta[1]*B)*X,
   kr <- K^{-1}*(A-eig*B)*X, being X <- V*pX[i_s..i_e-1], Y <- W*pY[i_s..i_e-1]
@@ -971,7 +923,7 @@ PETSC_STATIC_INLINE PetscErrorCode dvd_compute_n_rr(PetscInt i_s,PetscInt n,Pets
   pX,pY, the right and left eigenvectors of the projected system
   ld, the leading dimension of pX and pY
 */
-static PetscErrorCode dvd_improvex_jd_proj_uv_KZX(dvdDashboard *d,PetscInt i_s,PetscInt i_e,Vec *u,Vec *v,Vec *kr,PetscScalar *theta,PetscScalar *thetai,PetscScalar *pX,PetscScalar *pY,PetscInt ld)
+PetscErrorCode dvd_improvex_jd_proj_uv_KZX(dvdDashboard *d,PetscInt i_s,PetscInt i_e,Vec *u,Vec *v,Vec *kr,PetscScalar *theta,PetscScalar *thetai,PetscScalar *pX,PetscScalar *pY,PetscInt ld)
 {
   PetscErrorCode ierr;
   PetscInt       n = i_e-i_s,i;
@@ -1081,98 +1033,6 @@ static PetscErrorCode dvd_improvex_jd_proj_uv_KZX(dvdDashboard *d,PetscInt i_s,P
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "dvd_improvex_jd_proj_uv_KXX"
-/*
-  Compute: u <- K^{-1}*X, v <- X,
-  kr <- K^{-1}*(A-eig*B)*X, being X <- V*pX[i_s..i_e-1]
-  where
-  pX,pY, the right and left eigenvectors of the projected system
-  ld, the leading dimension of pX and pY
-*/
-static PetscErrorCode dvd_improvex_jd_proj_uv_KXX(dvdDashboard *d,PetscInt i_s,PetscInt i_e,Vec *u,Vec *v,Vec *kr,PetscScalar *theta,PetscScalar *thetai,PetscScalar *pX,PetscScalar *pY,PetscInt ld)
-{
-  PetscErrorCode ierr;
-  PetscInt       n = i_e - i_s,i;
-  PetscScalar    *b;
-  Vec            *Ax,*Bx,*r;
-  Mat            M;
-  BV             X;
-
-  PetscFunctionBegin;
-  ierr = BVDuplicateResize(d->eps->V,4,&X);CHKERRQ(ierr);
-  ierr = MatCreateSeqDense(PETSC_COMM_SELF,4,2,NULL,&M);CHKERRQ(ierr);
-  /* [v u] <- X(i) Y(i) */
-  ierr = dvd_improvex_compute_X(d,i_s,i_e,v,pX,ld);CHKERRQ(ierr);
-  for (i=i_s; i<i_e; ++i) {
-    ierr = BVMultVec(d->W?d->W:d->eps->V,1.0,0.0,u[i-i_s],&pY[ld*i]);CHKERRQ(ierr);
-  }
-
-  /* Bx <- B*X(i) */
-  ierr = SlepcVecPoolGetVecs(d->auxV,i_e-i_s,&r);CHKERRQ(ierr);
-  Bx = r;
-  if (d->BX) {
-    for (i=i_s; i<i_e; ++i) {
-      ierr = BVMultVec(d->BX,1.0,0.0,Bx[i-i_s],&pX[ld*i]);CHKERRQ(ierr);
-    }
-  } else {
-    if (d->B) {
-      for (i=0;i<n;i++) {
-        ierr = MatMult(d->B,v[i],Bx[i]);CHKERRQ(ierr);
-      }
-    } else Bx = v;
-  }
-
-  /* Ax <- A*X(i) */
-  Ax = kr;
-  for (i=i_s; i<i_e; ++i) {
-    ierr = BVMultVec(d->AX,1.0,0.0,Ax[i-i_s],&pX[ld*i]);CHKERRQ(ierr);
-  }
-
-  /* Recompute the eigenvalue */
-  ierr = dvd_compute_n_rr(i_s,n,d->eigr,d->eigi,u,Ax,Bx);CHKERRQ(ierr);
-
-  for (i=0;i<n;i++) {
-    if (d->eigi[i_s+i] == 0.0) {
-      /* kr <- Ax -eig*Bx */
-      ierr = VecAXPBY(kr[i],-d->eigr[i_s+i]/d->nX[i_s+i],1.0/d->nX[i_s+i],Bx[i]);CHKERRQ(ierr);
-    } else {
-      /* [kr_i kr_i+1 r_i r_i+1]*= [   1        0
-                                       0        1
-                                    -eigr_i -eigi_i
-                                     eigi_i -eigr_i] */
-      ierr = MatDenseGetArray(M,&b);CHKERRQ(ierr);
-      b[0] = b[5] = 1.0/d->nX[i_s+i];
-      b[2] = b[7] = -d->eigr[i_s+i]/d->nX[i_s+i];
-      b[6] = -(b[3] = d->eigi[i_s+i]/d->nX[i_s+i]);
-      b[1] = b[4] = 0.0;
-      ierr = MatDenseRestoreArray(M,&b);CHKERRQ(ierr);
-      ierr = BVInsertVec(X,0,kr[i]);CHKERRQ(ierr);
-      ierr = BVInsertVec(X,1,kr[i+1]);CHKERRQ(ierr);
-      ierr = BVInsertVec(X,2,r[i]);CHKERRQ(ierr);
-      ierr = BVInsertVec(X,3,r[i+1]);CHKERRQ(ierr);
-      ierr = BVSetActiveColumns(X,0,4);CHKERRQ(ierr);
-      ierr = BVMultInPlace(X,M,0,2);CHKERRQ(ierr);
-      ierr = BVCopyVec(X,0,kr[i]);CHKERRQ(ierr);
-      ierr = BVCopyVec(X,1,kr[i+1]);CHKERRQ(ierr);
-      i++;
-    }
-  }
-  for (i=0; i<n; i++) d->nX[i_s+i] = 1.0;
-
-  /* kr <- P*kr */
-  ierr = d->calcpairs_proj_res(d,i_s,i_e,r);CHKERRQ(ierr);
-  ierr = SlepcVecPoolRestoreVecs(d->auxV,i_e-i_s,&r);CHKERRQ(ierr);
-
-  /* u <- K^{-1} X(i) */
-  for (i=0;i<n;i++) {
-    ierr = d->improvex_precond(d,i_s+i,v[i],u[i]);CHKERRQ(ierr);
-  }
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "dvd_improvex_jd_lit_const_0"
 static PetscErrorCode dvd_improvex_jd_lit_const_0(dvdDashboard *d,PetscInt i,PetscScalar* theta,PetscScalar* thetai,PetscInt *maxits,PetscReal *tol)
 {
   dvdImprovex_jd *data = (dvdImprovex_jd*)d->improveX_data;
@@ -1203,8 +1063,6 @@ static PetscErrorCode dvd_improvex_jd_lit_const_0(dvdDashboard *d,PetscInt i,Pet
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "dvd_improvex_jd_lit_const"
 PetscErrorCode dvd_improvex_jd_lit_const(dvdDashboard *d,dvdBlackboard *b,PetscInt maxits,PetscReal tol,PetscReal fix)
 {
   dvdImprovex_jd  *data = (dvdImprovex_jd*)d->improveX_data;
@@ -1220,27 +1078,16 @@ PetscErrorCode dvd_improvex_jd_lit_const(dvdDashboard *d,dvdBlackboard *b,PetscI
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "dvd_improvex_jd_proj_uv"
-PetscErrorCode dvd_improvex_jd_proj_uv(dvdDashboard *d,dvdBlackboard *b,ProjType_t p)
+PetscErrorCode dvd_improvex_jd_proj_uv(dvdDashboard *d,dvdBlackboard *b)
 {
   PetscFunctionBegin;
   /* Setup the step */
   if (b->state >= DVD_STATE_CONF) {
-    switch (p) {
-    case DVD_PROJ_KXX:
-      d->improvex_jd_proj_uv = dvd_improvex_jd_proj_uv_KXX;
-      break;
-    case DVD_PROJ_KZX:
-      d->improvex_jd_proj_uv = dvd_improvex_jd_proj_uv_KZX;
-      break;
-    }
+    d->improvex_jd_proj_uv = dvd_improvex_jd_proj_uv_KZX;
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "dvd_improvex_compute_X"
 PetscErrorCode dvd_improvex_compute_X(dvdDashboard *d,PetscInt i_s,PetscInt i_e,Vec *u_,PetscScalar *pX,PetscInt ld)
 {
   PetscErrorCode ierr;
