@@ -1,5 +1,13 @@
 /*
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+   SLEPc - Scalable Library for Eigenvalue Problem Computations
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
+   This file is part of SLEPc.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+*/
+/*
    SLEPc eigensolver: "subspace"
 
    Method: Subspace Iteration
@@ -13,31 +21,10 @@
 
        [1] "Subspace Iteration in SLEPc", SLEPc Technical Report STR-3,
            available at http://slepc.upv.es.
-
-   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-   SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
-
-   This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
-   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
 #include <slepc/private/epsimpl.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSetUp_Subspace"
 PetscErrorCode EPSSetUp_Subspace(EPS eps)
 {
   PetscErrorCode ierr;
@@ -66,8 +53,6 @@ PetscErrorCode EPSSetUp_Subspace(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSubspaceFindGroup"
 /*
    EPSSubspaceFindGroup - Find a group of nearly equimodular eigenvalues, provided
    in arrays wr and wi, according to the tolerance grptol. Also the 2-norms
@@ -115,8 +100,6 @@ static PetscErrorCode EPSSubspaceFindGroup(PetscInt l,PetscInt m,PetscScalar *wr
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSubspaceResidualNorms"
 /*
    EPSSubspaceResidualNorms - Computes the column norms of residual vectors
    OP*V(1:n,l:m) - V*T(1:m,l:m), where, on entry, OP*V has been computed and
@@ -153,19 +136,16 @@ static PetscErrorCode EPSSubspaceResidualNorms(BV V,BV AV,PetscScalar *T,PetscIn
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSSolve_Subspace"
 PetscErrorCode EPSSolve_Subspace(EPS eps)
 {
   PetscErrorCode ierr;
-  Vec            v,av,w=eps->work[0];
-  Mat            H,Q;
+  Vec            w=eps->work[0];
+  Mat            H,Q,S;
   BV             AV;
   PetscInt       i,k,ld,ngrp,nogrp,*itrsd,*itrsdold;
   PetscInt       nxtsrr,idsrr,idort,nxtort,nv,ncv = eps->ncv,its;
   PetscScalar    *T;
   PetscReal      arsd,oarsd,ctr,octr,ae,oae,*rsd,norm,tcond=1.0;
-  PetscBool      breakdown;
   /* Parameters */
   PetscInt       init = 5;        /* Number of initial iterations */
   PetscReal      stpfac = 1.5;    /* Max num of iter before next SRR step */
@@ -181,6 +161,7 @@ PetscErrorCode EPSSolve_Subspace(EPS eps)
   ierr = PetscMalloc3(ncv,&rsd,ncv,&itrsd,ncv,&itrsdold);CHKERRQ(ierr);
   ierr = DSGetLeadingDimension(eps->ds,&ld);CHKERRQ(ierr);
   ierr = BVDuplicate(eps->V,&AV);CHKERRQ(ierr);
+  ierr = STGetOperator(eps->st,&S);CHKERRQ(ierr);
 
   for (i=0;i<ncv;i++) {
     rsd[i] = 0.0;
@@ -188,14 +169,9 @@ PetscErrorCode EPSSolve_Subspace(EPS eps)
   }
 
   /* Complete the initial basis with random vectors and orthonormalize them */
-  k = eps->nini;
-  while (k<ncv) {
+  for (k=eps->nini;k<ncv;k++) {
     ierr = BVSetRandomColumn(eps->V,k);CHKERRQ(ierr);
-    ierr = BVOrthogonalizeColumn(eps->V,k,NULL,&norm,&breakdown);CHKERRQ(ierr);
-    if (norm>0.0 && !breakdown) {
-      ierr = BVScaleColumn(eps->V,k,1.0/norm);CHKERRQ(ierr);
-      k++;
-    }
+    ierr = BVOrthonormalizeColumn(eps->V,k,PETSC_TRUE,NULL,NULL);CHKERRQ(ierr);
   }
 
   while (eps->reason == EPS_CONVERGED_ITERATING) {
@@ -207,17 +183,12 @@ PetscErrorCode EPSSolve_Subspace(EPS eps)
     ierr = EPSSubspaceFindGroup(eps->nconv,nv,eps->eigr,eps->eigi,rsd,grptol,&nogrp,&octr,&oae,&oarsd);CHKERRQ(ierr);
 
     /* AV(:,idx) = OP * V(:,idx) */
-    for (i=eps->nconv;i<nv;i++) {
-      ierr = BVGetColumn(eps->V,i,&v);CHKERRQ(ierr);
-      ierr = BVGetColumn(AV,i,&av);CHKERRQ(ierr);
-      ierr = STApply(eps->st,v,av);CHKERRQ(ierr);
-      ierr = BVRestoreColumn(eps->V,i,&v);CHKERRQ(ierr);
-      ierr = BVRestoreColumn(AV,i,&av);CHKERRQ(ierr);
-    }
+    ierr = BVSetActiveColumns(eps->V,eps->nconv,nv);CHKERRQ(ierr);
+    ierr = BVSetActiveColumns(AV,eps->nconv,nv);CHKERRQ(ierr);
+    ierr = BVMatMult(eps->V,S,AV);CHKERRQ(ierr);
 
     /* T(:,idx) = V' * AV(:,idx) */
     ierr = BVSetActiveColumns(eps->V,0,nv);CHKERRQ(ierr);
-    ierr = BVSetActiveColumns(AV,eps->nconv,nv);CHKERRQ(ierr);
     ierr = DSGetMat(eps->ds,DS_MAT_A,&H);CHKERRQ(ierr);
     ierr = BVDot(AV,eps->V,H);CHKERRQ(ierr);
     ierr = DSRestoreMat(eps->ds,DS_MAT_A,&H);CHKERRQ(ierr);
@@ -226,6 +197,7 @@ PetscErrorCode EPSSolve_Subspace(EPS eps)
     /* Solve projected problem */
     ierr = DSSolve(eps->ds,eps->eigr,eps->eigi);CHKERRQ(ierr);
     ierr = DSSort(eps->ds,eps->eigr,eps->eigi,NULL,NULL,NULL);CHKERRQ(ierr);
+    ierr = DSSynchronize(eps->ds,eps->eigr,eps->eigi);CHKERRQ(ierr);
 
     /* Update vectors V(:,idx) = V * U(:,idx) */
     ierr = DSGetMat(eps->ds,DS_MAT_Q,&Q);CHKERRQ(ierr);
@@ -285,33 +257,24 @@ PetscErrorCode EPSSolve_Subspace(EPS eps)
     /* Orthogonalization loop */
     do {
       while (its<nxtort) {
-
         /* A(:,idx) = OP*V(:,idx) with normalization */
+        ierr = BVMatMult(eps->V,S,AV);CHKERRQ(ierr);
+        ierr = BVCopy(AV,eps->V);CHKERRQ(ierr);
         for (i=eps->nconv;i<nv;i++) {
-          ierr = BVGetColumn(eps->V,i,&v);CHKERRQ(ierr);
-          ierr = STApply(eps->st,v,w);CHKERRQ(ierr);
-          ierr = VecCopy(w,v);CHKERRQ(ierr);
-          ierr = BVRestoreColumn(eps->V,i,&v);CHKERRQ(ierr);
           ierr = BVNormColumn(eps->V,i,NORM_INFINITY,&norm);CHKERRQ(ierr);
           ierr = BVScaleColumn(eps->V,i,1/norm);CHKERRQ(ierr);
         }
         its++;
       }
       /* Orthonormalize vectors */
-      for (i=eps->nconv;i<nv;i++) {
-        ierr = BVOrthogonalizeColumn(eps->V,i,NULL,&norm,&breakdown);CHKERRQ(ierr);
-        if (breakdown) {
-          ierr = BVSetRandomColumn(eps->V,i);CHKERRQ(ierr);
-          ierr = BVOrthogonalizeColumn(eps->V,i,NULL,&norm,&breakdown);CHKERRQ(ierr);
-        }
-        ierr = BVScaleColumn(eps->V,i,1/norm);CHKERRQ(ierr);
-      }
+      ierr = BVOrthogonalize(eps->V,NULL);CHKERRQ(ierr);
       nxtort = PetscMin(its+idort,nxtsrr);
     } while (its<nxtsrr);
   }
 
   ierr = PetscFree3(rsd,itrsd,itrsdold);CHKERRQ(ierr);
   ierr = BVDestroy(&AV);CHKERRQ(ierr);
+  ierr = MatDestroy(&S);CHKERRQ(ierr);
   /* truncate Schur decomposition and change the state to raw so that
      DSVectors() computes eigenvectors from scratch */
   ierr = DSSetDimensions(eps->ds,eps->nconv,0,0,0);CHKERRQ(ierr);
@@ -319,8 +282,6 @@ PetscErrorCode EPSSolve_Subspace(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSDestroy_Subspace"
 PetscErrorCode EPSDestroy_Subspace(EPS eps)
 {
   PetscErrorCode ierr;
@@ -330,16 +291,17 @@ PetscErrorCode EPSDestroy_Subspace(EPS eps)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EPSCreate_Subspace"
 PETSC_EXTERN PetscErrorCode EPSCreate_Subspace(EPS eps)
 {
   PetscFunctionBegin;
-  eps->ops->setup                = EPSSetUp_Subspace;
-  eps->ops->solve                = EPSSolve_Subspace;
-  eps->ops->destroy              = EPSDestroy_Subspace;
-  eps->ops->backtransform        = EPSBackTransform_Default;
-  eps->ops->computevectors       = EPSComputeVectors_Schur;
+  eps->useds = PETSC_TRUE;
+  eps->categ = EPS_CATEGORY_OTHER;
+
+  eps->ops->solve          = EPSSolve_Subspace;
+  eps->ops->setup          = EPSSetUp_Subspace;
+  eps->ops->destroy        = EPSDestroy_Subspace;
+  eps->ops->backtransform  = EPSBackTransform_Default;
+  eps->ops->computevectors = EPSComputeVectors_Schur;
   PetscFunctionReturn(0);
 }
 
