@@ -1,25 +1,15 @@
 /*
-     The basic MFN routines, Create, View, etc. are here.
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   Basic MFN routines
+*/
 
 #include <slepc/private/mfnimpl.h>      /*I "slepcmfn.h" I*/
 
@@ -28,8 +18,6 @@ PetscBool         MFNRegisterAllCalled = PETSC_FALSE;
 PetscClassId      MFN_CLASSID = 0;
 PetscLogEvent     MFN_SetUp = 0,MFN_Solve = 0;
 
-#undef __FUNCT__
-#define __FUNCT__ "MFNView"
 /*@C
    MFNView - Prints the MFN data structure.
 
@@ -93,8 +81,6 @@ PetscErrorCode MFNView(MFN mfn,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MFNReasonView"
 /*@C
    MFNReasonView - Displays the reason an MFN solve converged or diverged.
 
@@ -130,11 +116,9 @@ PetscErrorCode MFNReasonView(MFN mfn,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MFNReasonViewFromOptions"
 /*@
    MFNReasonViewFromOptions - Processes command line options to determine if/how
-   the MFN converged reason is to be viewed. 
+   the MFN converged reason is to be viewed.
 
    Collective on MFN
 
@@ -165,8 +149,6 @@ PetscErrorCode MFNReasonViewFromOptions(MFN mfn)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MFNCreate"
 /*@
    MFNCreate - Creates the default MFN context.
 
@@ -220,8 +202,6 @@ PetscErrorCode MFNCreate(MPI_Comm comm,MFN *outmfn)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MFNSetType"
 /*@C
    MFNSetType - Selects the particular solver to be used in the MFN object.
 
@@ -275,8 +255,6 @@ PetscErrorCode MFNSetType(MFN mfn,MFNType type)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MFNGetType"
 /*@C
    MFNGetType - Gets the MFN type as a string from the MFN object.
 
@@ -301,8 +279,6 @@ PetscErrorCode MFNGetType(MFN mfn,MFNType *type)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MFNRegister"
 /*@C
    MFNRegister - Adds a method to the matrix function solver package.
 
@@ -338,11 +314,9 @@ PetscErrorCode MFNRegister(const char *name,PetscErrorCode (*function)(MFN))
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MFNReset"
 /*@
-   MFNReset - Resets the MFN context to the setupcalled=0 state and removes any
-   allocated objects.
+   MFNReset - Resets the MFN context to the initial state (prior to setup)
+   and destroys any allocated Vecs and Mats.
 
    Collective on MFN
 
@@ -358,14 +332,17 @@ PetscErrorCode MFNReset(MFN mfn)
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(mfn,MFN_CLASSID,1);
+  if (mfn) PetscValidHeaderSpecific(mfn,MFN_CLASSID,1);
+  if (!mfn) PetscFunctionReturn(0);
   if (mfn->ops->reset) { ierr = (mfn->ops->reset)(mfn);CHKERRQ(ierr); }
+  ierr = MatDestroy(&mfn->A);CHKERRQ(ierr);
+  ierr = BVDestroy(&mfn->V);CHKERRQ(ierr);
+  ierr = VecDestroyVecs(mfn->nwork,&mfn->work);CHKERRQ(ierr);
+  mfn->nwork = 0;
   mfn->setupcalled = 0;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MFNDestroy"
 /*@
    MFNDestroy - Destroys the MFN context.
 
@@ -388,16 +365,12 @@ PetscErrorCode MFNDestroy(MFN *mfn)
   if (--((PetscObject)(*mfn))->refct > 0) { *mfn = 0; PetscFunctionReturn(0); }
   ierr = MFNReset(*mfn);CHKERRQ(ierr);
   if ((*mfn)->ops->destroy) { ierr = (*(*mfn)->ops->destroy)(*mfn);CHKERRQ(ierr); }
-  ierr = MatDestroy(&(*mfn)->A);CHKERRQ(ierr);
-  ierr = BVDestroy(&(*mfn)->V);CHKERRQ(ierr);
   ierr = FNDestroy(&(*mfn)->fn);CHKERRQ(ierr);
   ierr = MFNMonitorCancel(*mfn);CHKERRQ(ierr);
   ierr = PetscHeaderDestroy(mfn);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MFNSetBV"
 /*@
    MFNSetBV - Associates a basis vectors object to the matrix function solver.
 
@@ -430,8 +403,6 @@ PetscErrorCode MFNSetBV(MFN mfn,BV bv)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MFNGetBV"
 /*@
    MFNGetBV - Obtain the basis vectors object associated to the matrix
    function solver.
@@ -463,8 +434,6 @@ PetscErrorCode MFNGetBV(MFN mfn,BV *bv)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MFNSetFN"
 /*@
    MFNSetFN - Specifies the function to be computed.
 
@@ -497,8 +466,6 @@ PetscErrorCode MFNSetFN(MFN mfn,FN fn)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MFNGetFN"
 /*@
    MFNGetFN - Obtain the math function object associated to the MFN object.
 
