@@ -1,30 +1,18 @@
 /*
-      MFN routines related to the solution process.
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   MFN routines related to the solution process
+*/
 
 #include <slepc/private/mfnimpl.h>   /*I "slepcmfn.h" I*/
 
-#undef __FUNCT__
-#define __FUNCT__ "MFNSolve"
 /*@
    MFNSolve - Solves the matrix function problem. Given a vector b, the
    vector x = f(A)*b is returned.
@@ -75,7 +63,7 @@ PetscErrorCode MFNSolve(MFN mfn,Vec b,Vec x)
 
   /* check nonzero right-hand side */
   ierr = VecNorm(b,NORM_2,&mfn->bnorm);CHKERRQ(ierr);
-  if (!mfn->bnorm) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Cannot pass a zero b vector to MFNSolve()");
+  if (!mfn->bnorm) SETERRQ(PetscObjectComm((PetscObject)mfn),PETSC_ERR_ARG_WRONG,"Cannot pass a zero b vector to MFNSolve()");
 
   /* call solver */
   ierr = PetscLogEventBegin(MFN_Solve,mfn,b,x,0);CHKERRQ(ierr);
@@ -97,8 +85,6 @@ PetscErrorCode MFNSolve(MFN mfn,Vec b,Vec x)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MFNGetIterationNumber"
 /*@
    MFNGetIterationNumber - Gets the current iteration number. If the
    call to MFNSolve() is complete, then it returns the number of iterations
@@ -112,8 +98,6 @@ PetscErrorCode MFNSolve(MFN mfn,Vec b,Vec x)
    Output Parameter:
 .  its - number of iterations
 
-   Level: intermediate
-
    Note:
    During the i-th iteration this call returns i-1. If MFNSolve() is
    complete, then parameter "its" contains either the iteration number at
@@ -121,6 +105,8 @@ PetscErrorCode MFNSolve(MFN mfn,Vec b,Vec x)
    Call MFNGetConvergedReason() to determine if the solver converged or
    failed and why.
 
+   Level: intermediate
+
 .seealso: MFNGetConvergedReason(), MFNSetTolerances()
 @*/
 PetscErrorCode MFNGetIterationNumber(MFN mfn,PetscInt *its)
@@ -132,8 +118,6 @@ PetscErrorCode MFNGetIterationNumber(MFN mfn,PetscInt *its)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MFNGetConvergedReason"
 /*@
    MFNGetConvergedReason - Gets the reason why the MFNSolve() iteration was
    stopped.
