@@ -1,30 +1,18 @@
 /*
-     This file contains some simple default routines for common NEP operations.
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   Simple default routines for common NEP operations
+*/
 
 #include <slepc/private/nepimpl.h>     /*I "slepcnep.h" I*/
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPSetWorkVecs"
 /*@
    NEPSetWorkVecs - Sets a number of work vectors into a NEP object
 
@@ -57,8 +45,6 @@ PetscErrorCode NEPSetWorkVecs(NEP nep,PetscInt nw)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPGetDefaultShift"
 /*
   NEPGetDefaultShift - Return the value of sigma to start the nonlinear iteration.
  */
@@ -92,8 +78,6 @@ PetscErrorCode NEPGetDefaultShift(NEP nep,PetscScalar *sigma)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPConvergedRelative"
 /*
   NEPConvergedRelative - Checks convergence relative to the eigenvalue.
 */
@@ -107,8 +91,6 @@ PetscErrorCode NEPConvergedRelative(NEP nep,PetscScalar eigr,PetscScalar eigi,Pe
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPConvergedAbsolute"
 /*
   NEPConvergedAbsolute - Checks convergence absolutely.
 */
@@ -119,8 +101,6 @@ PetscErrorCode NEPConvergedAbsolute(NEP nep,PetscScalar eigr,PetscScalar eigi,Pe
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPConvergedNorm"
 /*
   NEPConvergedNorm - Checks convergence relative to the matrix norms.
 */
@@ -150,8 +130,6 @@ PetscErrorCode NEPConvergedNorm(NEP nep,PetscScalar eigr,PetscScalar eigi,PetscR
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPStoppingBasic"
 /*@C
    NEPStoppingBasic - Default routine to determine whether the outer eigensolver
    iteration must be stopped.
@@ -200,8 +178,6 @@ PetscErrorCode NEPStoppingBasic(NEP nep,PetscInt its,PetscInt max_it,PetscInt nc
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPComputeVectors_Schur"
 PetscErrorCode NEPComputeVectors_Schur(NEP nep)
 {
   PetscErrorCode ierr;
