@@ -1,52 +1,32 @@
 /*
-      NEP routines related to the solution process.
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+/*
+   NEP routines related to the solution process
+*/
 
 #include <slepc/private/nepimpl.h>       /*I "slepcnep.h" I*/
 #include <petscdraw.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPComputeVectors"
 PetscErrorCode NEPComputeVectors(NEP nep)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   NEPCheckSolved(nep,1);
-  switch (nep->state) {
-  case NEP_STATE_SOLVED:
-    if (nep->ops->computevectors) {
-      ierr = (*nep->ops->computevectors)(nep);CHKERRQ(ierr);
-    }
-    break;
-  default:
-    break;
+  if (nep->state==NEP_STATE_SOLVED && nep->ops->computevectors) {
+    ierr = (*nep->ops->computevectors)(nep);CHKERRQ(ierr);
   }
   nep->state = NEP_STATE_EIGENVECTORS;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPSolve"
 /*@
    NEPSolve - Solves the nonlinear eigensystem.
 
@@ -116,8 +96,6 @@ PetscErrorCode NEPSolve(NEP nep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPProjectOperator"
 /*@
    NEPProjectOperator - Computes the projection of the nonlinear operator.
 
@@ -163,8 +141,6 @@ PetscErrorCode NEPProjectOperator(NEP nep,PetscInt j0,PetscInt j1)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPApplyFunction"
 /*@
    NEPApplyFunction - Applies the nonlinear function T(lambda) to a given vector.
 
@@ -182,7 +158,7 @@ PetscErrorCode NEPProjectOperator(NEP nep,PetscInt j0,PetscInt j1)
 -  B   - optional preconditioning matrix
 
    Note:
-   If the nonlinear operator is represented in split form, the result 
+   If the nonlinear operator is represented in split form, the result
    y = T(lambda)*x is computed without building T(lambda) explicitly. In
    that case, parameters A and B are not used. Otherwise, the matrix
    T(lambda) is built and the effect is the same as a call to
@@ -221,8 +197,6 @@ PetscErrorCode NEPApplyFunction(NEP nep,PetscScalar lambda,Vec x,Vec v,Vec y,Mat
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPApplyJacobian"
 /*@
    NEPApplyJacobian - Applies the nonlinear Jacobian T'(lambda) to a given vector.
 
@@ -239,7 +213,7 @@ PetscErrorCode NEPApplyFunction(NEP nep,PetscScalar lambda,Vec x,Vec v,Vec y,Mat
 -  A   - Jacobian matrix
 
    Note:
-   If the nonlinear operator is represented in split form, the result 
+   If the nonlinear operator is represented in split form, the result
    y = T'(lambda)*x is computed without building T'(lambda) explicitly. In
    that case, parameter A is not used. Otherwise, the matrix
    T'(lambda) is built and the effect is the same as a call to
@@ -277,8 +251,6 @@ PetscErrorCode NEPApplyJacobian(NEP nep,PetscScalar lambda,Vec x,Vec v,Vec y,Mat
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPGetIterationNumber"
 /*@
    NEPGetIterationNumber - Gets the current iteration number. If the
    call to NEPSolve() is complete, then it returns the number of iterations
@@ -292,8 +264,6 @@ PetscErrorCode NEPApplyJacobian(NEP nep,PetscScalar lambda,Vec x,Vec v,Vec y,Mat
    Output Parameter:
 .  its - number of iterations
 
-   Level: intermediate
-
    Note:
    During the i-th iteration this call returns i-1. If NEPSolve() is
    complete, then parameter "its" contains either the iteration number at
@@ -301,6 +271,8 @@ PetscErrorCode NEPApplyJacobian(NEP nep,PetscScalar lambda,Vec x,Vec v,Vec y,Mat
    Call NEPGetConvergedReason() to determine if the solver converged or
    failed and why.
 
+   Level: intermediate
+
 .seealso: NEPGetConvergedReason(), NEPSetTolerances()
 @*/
 PetscErrorCode NEPGetIterationNumber(NEP nep,PetscInt *its)
@@ -312,8 +284,6 @@ PetscErrorCode NEPGetIterationNumber(NEP nep,PetscInt *its)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPGetConverged"
 /*@
    NEPGetConverged - Gets the number of converged eigenpairs.
 
@@ -342,8 +312,6 @@ PetscErrorCode NEPGetConverged(NEP nep,PetscInt *nconv)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPGetConvergedReason"
 /*@
    NEPGetConvergedReason - Gets the reason why the NEPSolve() iteration was
    stopped.
@@ -381,9 +349,7 @@ PetscErrorCode NEPGetConvergedReason(NEP nep,NEPConvergedReason *reason)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPGetEigenpair"
-/*@
+/*@C
    NEPGetEigenpair - Gets the i-th solution of the eigenproblem as computed by
    NEPSolve(). The solution consists in both the eigenvalue and the eigenvector.
 
@@ -464,8 +430,6 @@ PetscErrorCode NEPGetEigenpair(NEP nep,PetscInt i,PetscScalar *eigr,PetscScalar
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPGetErrorEstimate"
 /*@
    NEPGetErrorEstimate - Returns the error estimate associated to the i-th
    computed eigenpair.
@@ -493,13 +457,11 @@ PetscErrorCode NEPGetErrorEstimate(NEP nep,PetscInt i,PetscReal *errest)
   PetscValidHeaderSpecific(nep,NEP_CLASSID,1);
   PetscValidPointer(errest,3);
   NEPCheckSolved(nep,1);
-  if (i<0 || i>=nep->nconv) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Argument 2 out of range");
-  if (errest) *errest = nep->errest[nep->perm[i]];
+  if (i<0 || i>=nep->nconv) SETERRQ(PetscObjectComm((PetscObject)nep),PETSC_ERR_ARG_OUTOFRANGE,"Argument 2 out of range");
+  *errest = nep->errest[nep->perm[i]];
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPComputeResidualNorm_Private"
 /*
    NEPComputeResidualNorm_Private - Computes the norm of the residual vector
    associated with an eigenpair.
@@ -522,8 +484,6 @@ PetscErrorCode NEPComputeResidualNorm_Private(NEP nep,PetscScalar lambda,Vec x,V
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPComputeError"
 /*@
    NEPComputeError - Computes the error (based on the residual norm) associated
    with the i-th computed eigenpair.
@@ -582,7 +542,7 @@ PetscErrorCode NEPComputeError(NEP nep,PetscInt i,NEPErrorType type,PetscReal *e
   /* compute residual norms */
   ierr = NEPGetEigenpair(nep,i,&kr,&ki,xr,xi);CHKERRQ(ierr);
 #if !defined(PETSC_USE_COMPLEX)
-  if (ki) SETERRQ(PETSC_COMM_SELF,1,"Not implemented for complex eigenvalues with real scalars");
+  if (ki) SETERRQ(PetscObjectComm((PetscObject)nep),1,"Not implemented for complex eigenvalues with real scalars");
 #endif
   ierr = NEPComputeResidualNorm_Private(nep,kr,xr,nep->work,error);CHKERRQ(ierr);
   ierr = VecNorm(xr,NORM_2,&er);CHKERRQ(ierr);
@@ -620,8 +580,6 @@ PetscErrorCode NEPComputeError(NEP nep,PetscInt i,NEPErrorType type,PetscReal *e
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPComputeFunction"
 /*@
    NEPComputeFunction - Computes the function matrix T(lambda) that has been
    set with NEPSetFunction().
@@ -682,8 +640,6 @@ PetscErrorCode NEPComputeFunction(NEP nep,PetscScalar lambda,Mat A,Mat B)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NEPComputeJacobian"
 /*@
    NEPComputeJacobian - Computes the Jacobian matrix T'(lambda) that has been
    set with NEPSetJacobian().
