@@ -1,5 +1,13 @@
 /*
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+   SLEPc - Scalable Library for Eigenvalue Problem Computations
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
+   This file is part of SLEPc.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+*/
+/*
    SLEPc polynomial eigensolver: "jd"
 
    Method: Jacobi-Davidson
@@ -19,33 +27,46 @@
            "A Parallel Additive Schwarz Preconditioned Jacobi-Davidson
            Algorithm for Polynomial Eigenvalue Problems in Quantum Dot
            Simulation", J. Comput. Phys. 229(8):2932-2947, 2010.
-
-   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-   SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
-
-   This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
-   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
 #include <slepc/private/pepimpl.h>    /*I "slepcpep.h" I*/
-#include "pjdp.h"
 #include <slepcblaslapack.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPJDDuplicateBasis"
+typedef struct {
+  PetscReal   keep;          /* restart parameter */
+  BV          V;             /* work basis vectors to store the search space */
+  BV          W;             /* work basis vectors to store the test space */
+  BV          *TV;           /* work basis vectors to store T*V (each TV[i] is the coefficient for \lambda^i of T*V for the extended T) */
+  BV          *AX;           /* work basis vectors to store A_i*X for locked eigenvectors */
+  BV          X;             /* locked eigenvectors */
+  PetscScalar *T;            /* matrix of the invariant pair */
+  PetscScalar *Tj;           /* matrix containing the powers of the invariant pair matrix */
+  PetscScalar *XpX;          /* X^H*X */
+  PC          pcshell;       /* preconditioner including basic precond+projector */
+  Mat         Pshell;        /* auxiliary shell matrix */
+  PetscInt    nconv;         /* number of locked vectors in the invariant pair */
+} PEP_JD;
+
+typedef struct {
+  PC          pc;            /* basic preconditioner */
+  Vec         Bp;            /* preconditioned residual of derivative polynomial, B\p */
+  Vec         u;             /* Ritz vector */
+  PetscScalar gamma;         /* precomputed scalar u'*B\p */
+  PetscScalar *M;
+  PetscScalar *ps;
+  PetscInt    ld;
+  Vec         *work;
+  BV          X;
+  PetscInt    n;
+} PEP_JD_PCSHELL;
+
+typedef struct {
+  Mat         P;             /*  */
+  PEP         pep;
+  Vec         *work;
+  PetscScalar theta;
+} PEP_JD_MATSHELL;
+
 /*
    Duplicate and resize auxiliary basis
 */
@@ -79,15 +100,12 @@ static PetscErrorCode PEPJDDuplicateBasis(PEP pep,BV *basis)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSetUp_JD"
 PetscErrorCode PEPSetUp_JD(PEP pep)
 {
   PetscErrorCode ierr;
   PEP_JD         *pjd = (PEP_JD*)pep->data;
   PetscBool      isprecond,flg;
   PetscInt       i;
-  KSP            ksp;
 
   PetscFunctionBegin;
   pep->lineariz = PETSC_FALSE;
@@ -96,10 +114,6 @@ PetscErrorCode PEPSetUp_JD(PEP pep)
   if (!pep->which) pep->which = PEP_TARGET_MAGNITUDE;
   if (pep->which != PEP_TARGET_MAGNITUDE) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_SUP,"PEPJD only supports which=target_magnitude");;
 
-  /* Set STPRECOND as the default ST */
-  if (!((PetscObject)pep->st)->type_name) {
-    ierr = STSetType(pep->st,STPRECOND);CHKERRQ(ierr);
-  }
   ierr = PetscObjectTypeCompare((PetscObject)pep->st,STPRECOND,&isprecond);CHKERRQ(ierr);
   if (!isprecond) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_SUP,"JD only works with PRECOND spectral transformation");
 
@@ -107,13 +121,6 @@ PetscErrorCode PEPSetUp_JD(PEP pep)
   ierr = STGetTransform(pep->st,&flg);CHKERRQ(ierr);
   if (flg) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_SUP,"Solver requires the ST transformation flag unset, see STSetTransform()");
 
-  /* Set the default options of the KSP */
-  ierr = STGetKSP(pep->st,&ksp);CHKERRQ(ierr);
-  if (!((PetscObject)ksp)->type_name) {
-    ierr = KSPSetType(ksp,KSPBCGSL);CHKERRQ(ierr);
-    ierr = KSPSetTolerances(ksp,1e-5,PETSC_DEFAULT,PETSC_DEFAULT,100);CHKERRQ(ierr);
-  }
-
   if (!pjd->keep) pjd->keep = 0.5;
 
   ierr = PEPAllocateSolution(pep,0);CHKERRQ(ierr);
@@ -138,8 +145,6 @@ PetscErrorCode PEPSetUp_JD(PEP pep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPJDUpdateTV"
 /*
    Updates columns (low to (high-1)) of TV[i]
 */
@@ -158,7 +163,7 @@ static PetscErrorCode PEPJDUpdateTV(PEP pep,PetscInt low,PetscInt high,Vec *w)
   ierr = PetscMalloc3(nconv,&tt,nconv,&x2,nconv,&xx);CHKERRQ(ierr);
   ierr = MPI_Comm_rank(PetscObjectComm((PetscObject)pep),&rk);CHKERRQ(ierr);
   ierr = MPI_Comm_size(PetscObjectComm((PetscObject)pep),&np);CHKERRQ(ierr);
-  ierr = BVGetSizes(pep->V,&nloc,NULL,NULL);CHKERRQ(ierr); 
+  ierr = BVGetSizes(pep->V,&nloc,NULL,NULL);CHKERRQ(ierr);
   t1 = w[0];
   t2 = w[1];
   for (col=low;col<high;col++) {
@@ -198,7 +203,7 @@ static PetscErrorCode PEPJDUpdateTV(PEP pep,PetscInt low,PetscInt high,Vec *w)
       }
       ierr = VecResetArray(t2);CHKERRQ(ierr);
       ierr = VecRestoreArray(v2,&array2);CHKERRQ(ierr);
-      ierr = BVRestoreColumn(pjd->TV[pp],col,&v2);CHKERRQ(ierr);        
+      ierr = BVRestoreColumn(pjd->TV[pp],col,&v2);CHKERRQ(ierr);
     }
     ierr = VecResetArray(t1);CHKERRQ(ierr);
     ierr = VecRestoreArray(v1,&array1);CHKERRQ(ierr);
@@ -208,16 +213,14 @@ static PetscErrorCode PEPJDUpdateTV(PEP pep,PetscInt low,PetscInt high,Vec *w)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPJDOrthogonalize"
 /*
    RRQR of X. Xin*P=Xou*R. Rank of R is rk
 */
 static PetscErrorCode PEPJDOrthogonalize(PetscInt row,PetscInt col,PetscScalar *X,PetscInt ldx,PetscInt *rk,PetscInt *P,PetscScalar *R,PetscInt ldr)
 {
-#if defined(SLEPC_MISSING_LAPACK_GEQP3)
+#if defined(SLEPC_MISSING_LAPACK_GEQP3) || defined(PETSC_MISSING_LAPACK_ORGQR)
   PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"GEQP3 - Lapack routine is unavailable");
+  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"GEQP3/QRGQR - Lapack routines are unavailable");
 #else
   PetscErrorCode ierr;
   PetscInt       i,j,n,r;
@@ -242,13 +245,13 @@ static PetscErrorCode PEPJDOrthogonalize(PetscInt row,PetscInt col,PetscScalar *
 #else
   PetscStackCallBLAS("LAPACKgeqp3",LAPACKgeqp3_(&row_,&col_,X,&ldx_,p,tau,work,&lwork,&info));
 #endif
-  if (info) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"Error in Lapack xGEQP3 %d",info);
+  SlepcCheckLapackInfo("geqp3",info);
   if (P) for (i=0;i<col;i++) P[i] = p[i];
 
   /* rank computation */
   tol = PetscMax(row,col)*PETSC_MACHINE_EPSILON*PetscAbsScalar(X[0]);
   r = 1;
-  for (i=1;i<n;i++) { 
+  for (i=1;i<n;i++) {
     if (PetscAbsScalar(X[i+ldx*i])>tol) r++;
     else break;
   }
@@ -262,14 +265,12 @@ static PetscErrorCode PEPJDOrthogonalize(PetscInt row,PetscInt col,PetscScalar *
      }
   }
   PetscStackCallBLAS("LAPACKungqr",LAPACKungqr_(&row_,&n_,&n_,X,&ldx_,tau,work,&lwork,&info));
-  if (info) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"Error in Lapack xORGQR %d",info);
+  SlepcCheckLapackInfo("ungqr",info);
   ierr = PetscFree4(p,tau,work,rwork);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 #endif
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPJDExtendedPCApply"
 /*
    Application of extended preconditioner
 */
@@ -294,7 +295,7 @@ static PetscErrorCode PEPJDExtendedPCApply(PC pc,Vec x,Vec y)
     ierr = MPI_Comm_rank(PetscObjectComm((PetscObject)pc),&rk);CHKERRQ(ierr);
     ierr = MPI_Comm_size(PetscObjectComm((PetscObject)pc),&np);CHKERRQ(ierr);
     if (rk==np-1) {
-      ierr = VecGetLocalSize(ctx->work[0],&nloc);CHKERRQ(ierr); 
+      ierr = VecGetLocalSize(ctx->work[0],&nloc);CHKERRQ(ierr);
       ierr = VecGetArrayRead(x,&array1);CHKERRQ(ierr);
       for (i=0;i<n;i++) x2[i] = array1[nloc+i];
       ierr = VecRestoreArrayRead(x,&array1);CHKERRQ(ierr);
@@ -330,8 +331,6 @@ static PetscErrorCode PEPJDExtendedPCApply(PC pc,Vec x,Vec y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCShellApply_PEPJD"
 /*
    Application of shell preconditioner:
       y = B\x - eta*B\p,  with eta = (u'*B\x)/(u'*B\p)
@@ -351,14 +350,12 @@ static PetscErrorCode PCShellApply_PEPJD(PC pc,Vec x,Vec y)
   /* Compute eta = u'*y / u'*Bp */
   ierr = VecDot(y,ctx->u,&eta);CHKERRQ(ierr);
   eta /= ctx->gamma;
-  
+
   /* y = y - eta*Bp */
-  ierr = VecAXPY(y,-eta,ctx->Bp);CHKERRQ(ierr); 
+  ierr = VecAXPY(y,-eta,ctx->Bp);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPJDCopyToExtendedVec"
 static PetscErrorCode PEPJDCopyToExtendedVec(PEP pep,Vec v,PetscScalar *a,PetscInt na,PetscInt off,Vec vex,PetscBool back)
 {
   PetscErrorCode ierr;
@@ -399,8 +396,6 @@ static PetscErrorCode PEPJDCopyToExtendedVec(PEP pep,Vec v,PetscScalar *a,PetscI
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPJDComputePResidual"
 static PetscErrorCode PEPJDComputePResidual(PEP pep,Vec u,PetscScalar theta,Vec p,Vec *work)
 {
   PEP_JD         *pjd = (PEP_JD*)pep->data;
@@ -417,7 +412,7 @@ static PetscErrorCode PEPJDComputePResidual(PEP pep,Vec u,PetscScalar theta,Vec
     ierr = MPI_Comm_rank(PetscObjectComm((PetscObject)pep),&rk);CHKERRQ(ierr);
     ierr = MPI_Comm_size(PetscObjectComm((PetscObject)pep),&np);CHKERRQ(ierr);
     if (rk==np-1) {
-      ierr = BVGetSizes(pep->V,&nloc,NULL,NULL);CHKERRQ(ierr); 
+      ierr = BVGetSizes(pep->V,&nloc,NULL,NULL);CHKERRQ(ierr);
       ierr = VecGetArray(u,&array1);CHKERRQ(ierr);
       for (i=0;i<nconv;i++) x2[i] = array1[nloc+i];
       ierr = VecRestoreArray(u,&array1);CHKERRQ(ierr);
@@ -475,8 +470,6 @@ static PetscErrorCode PEPJDComputePResidual(PEP pep,Vec u,PetscScalar theta,Vec
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPJDProcessInitialSpace"
 static PetscErrorCode PEPJDProcessInitialSpace(PEP pep,Vec *w)
 {
   PEP_JD         *pjd = (PEP_JD*)pep->data;
@@ -504,15 +497,11 @@ static PetscErrorCode PEPJDProcessInitialSpace(PEP pep,Vec *w)
     ierr = BVRestoreColumn(pjd->V,0,&vg);CHKERRQ(ierr);
     ierr = BVNormColumn(pjd->W,0,NORM_2,&norm);CHKERRQ(ierr);
     ierr = BVScaleColumn(pjd->W,0,1.0/norm);CHKERRQ(ierr);
-  } else {
-   SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"TO DO");
-  }
+  } else SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Support for initial vectors not implemented yet");
   ierr = PetscFree(tt);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPJDShellMatMult"
 static PetscErrorCode PEPJDShellMatMult(Mat P,Vec x,Vec y)
 {
   PetscErrorCode    ierr;
@@ -538,7 +527,7 @@ static PetscErrorCode PEPJDShellMatMult(Mat P,Vec x,Vec y)
     ierr = MPI_Comm_rank(PetscObjectComm((PetscObject)P),&rk);CHKERRQ(ierr);
     ierr = MPI_Comm_size(PetscObjectComm((PetscObject)P),&np);CHKERRQ(ierr);
     if (rk==np-1) {
-      ierr = BVGetSizes(matctx->pep->V,&nloc,NULL,NULL);CHKERRQ(ierr); 
+      ierr = BVGetSizes(matctx->pep->V,&nloc,NULL,NULL);CHKERRQ(ierr);
       ierr = VecGetArrayRead(x,&array1);CHKERRQ(ierr);
       for (i=0;i<nconv;i++) x2[i] = array1[nloc+i];
       ierr = VecRestoreArrayRead(x,&array1);CHKERRQ(ierr);
@@ -565,7 +554,7 @@ static PetscErrorCode PEPJDShellMatMult(Mat P,Vec x,Vec y)
       for (j=0;j<nconv;j++) q[j] += fact*x2[j];
       fact *= theta;
     }
-    ierr = BVSetActiveColumns(pjd->X,0,nconv);CHKERRQ(ierr);    
+    ierr = BVSetActiveColumns(pjd->X,0,nconv);CHKERRQ(ierr);
     ierr = BVDotVec(pjd->X,tx,xx);CHKERRQ(ierr);
     if (rk==np-1) {
       y2 = array2+nloc;
@@ -590,8 +579,6 @@ static PetscErrorCode PEPJDShellMatMult(Mat P,Vec x,Vec y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPJDCreateShellPC"
 static PetscErrorCode PEPJDCreateShellPC(PEP pep)
 {
   PEP_JD          *pjd = (PEP_JD*)pep->data;
@@ -635,24 +622,22 @@ static PetscErrorCode PEPJDCreateShellPC(PEP pep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPJDUpdateExtendedPC"
 static PetscErrorCode PEPJDUpdateExtendedPC(PEP pep,PetscScalar theta)
 {
+#if defined(PETSC_MISSING_LAPACK_GESVD) || defined(PETSC_MISSING_LAPACK_GETRI) || defined(PETSC_MISSING_LAPACK_GETRF)
+  PetscFunctionBegin;
+  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"GESVD/GETRI/GETRF - Lapack routines are unavailable");
+#else
   PetscErrorCode ierr;
   PEP_JD         *pjd = (PEP_JD*)pep->data;
-  PEP_JD_PCSHELL *pcctx;  
+  PEP_JD_PCSHELL *pcctx;
   PetscInt       i,j,k,n=pjd->nconv,ld=pep->nev,deg=pep->nmat-1;
   PetscScalar    fact,*M,*ps,*work,*U,*V,*S,sone=1.0,zero=0.0;
   PetscReal      tol,maxeig=0.0,*sg,*rwork;
   PetscBLASInt   n_,info,ld_,*p,lw_,rk=0;
 
   PetscFunctionBegin;
-#if defined(PETSC_MISSING_LAPACK_GESVD) || defined(PETSC_MISSING_LAPACK_GETRI) || defined(PETSC_MISSING_LAPACK_GETRF)
-  PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"GESVD/GETRI/GETRF - Lapack routine is unavailable");
-#else
-  if (n) { 
+  if (n) {
     ierr = PCShellGetContext(pjd->pcshell,(void**)&pcctx);CHKERRQ(ierr);
     pcctx->n = n;
     M  = pcctx->M;
@@ -672,6 +657,7 @@ static PetscErrorCode PEPJDUpdateExtendedPC(PEP pep,PetscScalar theta)
 #else
     PetscStackCallBLAS("LAPACKgesvd",LAPACKgesvd_("S","S",&n_,&n_,S,&n_,sg,U,&n_,V,&n_,work,&lw_,rwork,&info));
 #endif
+    SlepcCheckLapackInfo("gesvd",info);
     for (i=0;i<n;i++) maxeig = PetscMax(maxeig,sg[i]);
     tol = 10*PETSC_MACHINE_EPSILON*n*maxeig;
     for (j=0;j<n;j++) {
@@ -697,21 +683,21 @@ static PetscErrorCode PEPJDUpdateExtendedPC(PEP pep,PetscScalar theta)
     }
     /* inverse */
     PetscStackCallBLAS("LAPACKgetrf",LAPACKgetrf_(&n_,&n_,M,&ld_,p,&info));
+    SlepcCheckLapackInfo("getrf",info);
     PetscStackCallBLAS("LAPACKgetri",LAPACKgetri_(&n_,M,&ld_,p,work,&n_,&info));
+    SlepcCheckLapackInfo("getri",info);
     ierr = PetscFree7(U,V,S,sg,work,rwork,p);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 #endif
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPJDPCMatSetUp"
 static PetscErrorCode PEPJDPCMatSetUp(PEP pep,PetscScalar theta)
 {
   PetscErrorCode  ierr;
   PEP_JD          *pjd = (PEP_JD*)pep->data;
   PEP_JD_MATSHELL *matctx;
-  PEP_JD_PCSHELL  *pcctx;  
+  PEP_JD_PCSHELL  *pcctx;
   MatStructure    str;
   PetscScalar     t;
   PetscInt        i;
@@ -732,10 +718,12 @@ static PetscErrorCode PEPJDPCMatSetUp(PEP pep,PetscScalar theta)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPJDEigenvectors"
 static PetscErrorCode PEPJDEigenvectors(PEP pep)
 {
+#if defined(SLEPC_MISSING_LAPACK_TREVC)
+  PetscFunctionBegin;
+  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"TREVC - Lapack routine is unavailable");
+#else
   PetscErrorCode ierr;
   PEP_JD         *pjd = (PEP_JD*)pep->data;
   PetscBLASInt   ld,nconv,info,nc;
@@ -743,12 +731,8 @@ static PetscErrorCode PEPJDEigenvectors(PEP pep)
   PetscReal      *wr,norm;
   PetscInt       i;
   Mat            U;
- 
+
   PetscFunctionBegin;
-#if defined(SLEPC_MISSING_LAPACK_TREVC)
-  PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"TREVC - Lapack routine is unavailable");
-#else
   ierr = PetscMalloc3(pjd->nconv*pjd->nconv,&Z,3*pep->nev,&wr,2*pep->nev,&w);CHKERRQ(ierr);
   ierr = PetscBLASIntCast(pep->nev,&ld);CHKERRQ(ierr);
   ierr = PetscBLASIntCast(pjd->nconv,&nconv);CHKERRQ(ierr);
@@ -757,12 +741,13 @@ static PetscErrorCode PEPJDEigenvectors(PEP pep)
 #else
   PetscStackCallBLAS("LAPACKtrevc",LAPACKtrevc_("R","A",NULL,&nconv,pjd->T,&ld,NULL,&nconv,Z,&nconv,&nconv,&nc,w,wr,&info));
 #endif
+  SlepcCheckLapackInfo("trevc",info);
   ierr = MatCreateSeqDense(PETSC_COMM_SELF,nconv,nconv,Z,&U);CHKERRQ(ierr);
   ierr = BVSetActiveColumns(pjd->X,0,pjd->nconv);CHKERRQ(ierr);
   ierr = BVMultInPlace(pjd->X,U,0,pjd->nconv);CHKERRQ(ierr);
   for (i=0;i<pjd->nconv;i++) {
     ierr = BVNormColumn(pjd->X,i,NORM_2,&norm);CHKERRQ(ierr);
-    ierr = BVScaleColumn(pjd->X,i,1.0/norm);CHKERRQ(ierr);  
+    ierr = BVScaleColumn(pjd->X,i,1.0/norm);CHKERRQ(ierr);
   }
   ierr = MatDestroy(&U);CHKERRQ(ierr);
   ierr = PetscFree3(Z,wr,w);CHKERRQ(ierr);
@@ -770,23 +755,21 @@ static PetscErrorCode PEPJDEigenvectors(PEP pep)
 #endif
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPJDLockConverged"
 static PetscErrorCode PEPJDLockConverged(PEP pep,PetscInt *nv)
 {
-  PetscErrorCode    ierr;
-  PEP_JD            *pjd = (PEP_JD*)pep->data;
-  PetscInt          j,i,ldds,rk=0,*P,nvv=*nv;
-  Vec               v,x;
-  PetscBLASInt      n,ld,rk_,nv_,info,one=1;
-  PetscScalar       sone=1.0,*Tj,*R,*r,*tt,*pX;
-  Mat               X;
-
-  PetscFunctionBegin;
 #if defined(SLEPC_MISSING_LAPACK_TRTRI)
   PetscFunctionBegin;
   SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"TRTRI - Lapack routine is unavailable");
 #else
+  PetscErrorCode ierr;
+  PEP_JD         *pjd = (PEP_JD*)pep->data;
+  PetscInt       j,i,ldds,rk=0,*P,nvv=*nv;
+  Vec            v,x;
+  PetscBLASInt   n,ld,rk_,nv_,info,one=1;
+  PetscScalar    sone=1.0,*Tj,*R,*r,*tt,*pX;
+  Mat            X;
+
+  PetscFunctionBegin;
   /* update AX and XpX */
   ierr = BVGetColumn(pjd->X,pjd->nconv-1,&x);CHKERRQ(ierr);
   for (j=0;j<pep->nmat;j++) {
@@ -799,7 +782,7 @@ static PetscErrorCode PEPJDLockConverged(PEP pep,PetscInt *nv)
   ierr = BVDotColumn(pjd->X,(pjd->nconv-1),pjd->XpX+(pjd->nconv-1)*(pep->nev));CHKERRQ(ierr);
   pjd->XpX[(pjd->nconv-1)*(1+pep->nev)] = 1.0;
   for (j=0;j<pjd->nconv-1;j++) pjd->XpX[j*(pep->nev)+pjd->nconv-1] = PetscConj(pjd->XpX[(pjd->nconv-1)*(pep->nev)+j]);
-  
+
   /* Compute powers of T */
   ierr = PetscBLASIntCast(pjd->nconv,&n);CHKERRQ(ierr);
   ierr = PetscBLASIntCast(pep->nev,&ld);CHKERRQ(ierr);
@@ -823,13 +806,13 @@ static PetscErrorCode PEPJDLockConverged(PEP pep,PetscInt *nv)
   ierr = PetscBLASIntCast(rk,&rk_);CHKERRQ(ierr);
   ierr = PetscBLASIntCast(nvv,&nv_);CHKERRQ(ierr);
   PetscStackCallBLAS("LAPACKtrtri",LAPACKtrtri_("U","N",&rk_,R,&nv_,&info));
-  if (info) SETERRQ1(PETSC_COMM_SELF,1,"Error in xTRTRI, info=%D",(PetscInt)info);
+  SlepcCheckLapackInfo("trtri",info);
   PetscStackCallBLAS("BLAStrmv",BLAStrmv_("U","C","N",&rk_,R,&nv_,r,&one));
   for (i=0;i<rk;i++) r[i] = PetscConj(r[i]); /* revert */
   ierr = BVSetActiveColumns(pjd->V,0,nvv);CHKERRQ(ierr);
   for (j=0;j<rk-1;j++) {
     ierr = PetscMemcpy(R+j*nvv,pX+(j+1)*ldds,nvv*sizeof(PetscScalar));CHKERRQ(ierr);
-  } 
+  }
   ierr = DSRestoreArray(pep->ds,DS_MAT_X,&pX);CHKERRQ(ierr);
   ierr = MatCreateSeqDense(PETSC_COMM_SELF,nvv,rk-1,R,&X);CHKERRQ(ierr);
   ierr = BVMultInPlace(pjd->V,X,0,rk-1);CHKERRQ(ierr);
@@ -840,7 +823,7 @@ static PetscErrorCode PEPJDLockConverged(PEP pep,PetscInt *nv)
     ierr = PEPJDCopyToExtendedVec(pep,NULL,r+j,1,pjd->nconv-1,v,PETSC_FALSE);CHKERRQ(ierr);
     ierr = BVRestoreColumn(pjd->V,j,&v);CHKERRQ(ierr);
   }
-  ierr = BVOrthogonalize(pjd->V,NULL);CHKERRQ(ierr); 
+  ierr = BVOrthogonalize(pjd->V,NULL);CHKERRQ(ierr);
   for (j=0;j<rk-1;j++) {
     ierr = BVGetColumn(pjd->W,j,&v);CHKERRQ(ierr);
     ierr = PEPJDCopyToExtendedVec(pep,NULL,tt,pep->nev-1,0,v,PETSC_FALSE);CHKERRQ(ierr);
@@ -852,8 +835,6 @@ static PetscErrorCode PEPJDLockConverged(PEP pep,PetscInt *nv)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPSolve_JD"
 PetscErrorCode PEPSolve_JD(PEP pep)
 {
   PetscErrorCode  ierr;
@@ -909,6 +890,7 @@ PetscErrorCode PEPSolve_JD(PEP pep)
       ierr = DSSetState(pep->ds,DS_STATE_RAW);CHKERRQ(ierr);
       ierr = DSSolve(pep->ds,pep->eigr+pep->nconv,pep->eigi+pep->nconv);CHKERRQ(ierr);
       ierr = DSSort(pep->ds,pep->eigr+pep->nconv,pep->eigi+pep->nconv,NULL,NULL,NULL);CHKERRQ(ierr);
+      ierr = DSSynchronize(pep->ds,pep->eigr+pep->nconv,pep->eigi+pep->nconv);CHKERRQ(ierr);
       theta = pep->eigr[0];
 #if !defined(PETSC_USE_COMPLEX)
       if (PetscAbsScalar(pep->eigi[pep->nconv])!=0.0) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"PJD solver not implemented for complex Ritz values in real arithmetic");
@@ -931,7 +913,7 @@ PetscErrorCode PEPSolve_JD(PEP pep)
       pcctx->work  = ww;
     }
     ierr = PEPJDPCMatSetUp(pep,theta);CHKERRQ(ierr);
-    
+
     /* Compute r and r' */
     ierr = MatMult(pjd->Pshell,u,r);CHKERRQ(ierr);
     ierr = PEPJDComputePResidual(pep,u,theta,p,ww);CHKERRQ(ierr);
@@ -979,10 +961,10 @@ PetscErrorCode PEPSolve_JD(PEP pep)
       ierr = DSRestoreArray(pep->ds,DS_MAT_Y,&pX);CHKERRQ(ierr);
       ierr = DSGetMat(pep->ds,DS_MAT_X,&X);CHKERRQ(ierr);
       ierr = BVMultInPlace(pjd->V,X,pep->nconv,minv);CHKERRQ(ierr);
-      ierr = DSRestoreMat(pep->ds,DS_MAT_X,&X);CHKERRQ(ierr);
+      ierr = MatDestroy(&X);CHKERRQ(ierr);
       ierr = DSGetMat(pep->ds,DS_MAT_Y,&Y);CHKERRQ(ierr);
       ierr = BVMultInPlace(pjd->W,Y,pep->nconv,minv);CHKERRQ(ierr);
-      ierr = DSRestoreMat(pep->ds,DS_MAT_Y,&Y);CHKERRQ(ierr);
+      ierr = MatDestroy(&Y);CHKERRQ(ierr);
       nv = minv;
       flgre = PETSC_TRUE;
     } else {
@@ -1018,12 +1000,12 @@ PetscErrorCode PEPSolve_JD(PEP pep)
       ierr = BVGetColumn(pjd->X,k,&v);CHKERRQ(ierr);
       ierr = BVInsertVec(pep->V,k,v);CHKERRQ(ierr);
       ierr = BVRestoreColumn(pjd->X,k,&v);CHKERRQ(ierr);
-      pep->eigr[k] = pjd->T[(pep->nev+1)*k]; 
-      pep->eigi[k] = 0.0; 
+      pep->eigr[k] = pjd->T[(pep->nev+1)*k];
+      pep->eigi[k] = 0.0;
     }
-    ierr = PetscFree2(pcctx->M,pcctx->ps);CHKERRQ(ierr); 
+    ierr = PetscFree2(pcctx->M,pcctx->ps);CHKERRQ(ierr);
   }
-  pep->nconv = pjd->nconv; 
+  pep->nconv = pjd->nconv;
   ierr = KSPSetPC(ksp,pcctx->pc);CHKERRQ(ierr);
   ierr = MatDestroy(&matctx->P);CHKERRQ(ierr);
   ierr = VecDestroy(&pcctx->Bp);CHKERRQ(ierr);
@@ -1039,8 +1021,135 @@ PetscErrorCode PEPSolve_JD(PEP pep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPReset_JD"
+PetscErrorCode PEPJDSetRestart_JD(PEP pep,PetscReal keep)
+{
+  PEP_JD *pjd = (PEP_JD*)pep->data;
+
+  PetscFunctionBegin;
+  if (keep==PETSC_DEFAULT) pjd->keep = 0.5;
+  else {
+    if (keep<0.1 || keep>0.9) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_ARG_OUTOFRANGE,"The keep argument must be in the range [0.1,0.9]");
+    pjd->keep = keep;
+  }
+  PetscFunctionReturn(0);
+}
+
+/*@
+   PEPJDSetRestart - Sets the restart parameter for the Jacobi-Davidson
+   method, in particular the proportion of basis vectors that must be kept
+   after restart.
+
+   Logically Collective on PEP
+
+   Input Parameters:
++  pep  - the eigenproblem solver context
+-  keep - the number of vectors to be kept at restart
+
+   Options Database Key:
+.  -pep_jd_restart - Sets the restart parameter
+
+   Notes:
+   Allowed values are in the range [0.1,0.9]. The default is 0.5.
+
+   Level: advanced
+
+.seealso: PEPJDGetRestart()
+@*/
+PetscErrorCode PEPJDSetRestart(PEP pep,PetscReal keep)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(pep,PEP_CLASSID,1);
+  PetscValidLogicalCollectiveReal(pep,keep,2);
+  ierr = PetscTryMethod(pep,"PEPJDSetRestart_C",(PEP,PetscReal),(pep,keep));CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+PetscErrorCode PEPJDGetRestart_JD(PEP pep,PetscReal *keep)
+{
+  PEP_JD *pjd = (PEP_JD*)pep->data;
+
+  PetscFunctionBegin;
+  *keep = pjd->keep;
+  PetscFunctionReturn(0);
+}
+
+/*@
+   PEPJDGetRestart - Gets the restart parameter used in the Jacobi-Davidson method.
+
+   Not Collective
+
+   Input Parameter:
+.  pep - the eigenproblem solver context
+
+   Output Parameter:
+.  keep - the restart parameter
+
+   Level: advanced
+
+.seealso: PEPJDSetRestart()
+@*/
+PetscErrorCode PEPJDGetRestart(PEP pep,PetscReal *keep)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(pep,PEP_CLASSID,1);
+  PetscValidPointer(keep,2);
+  ierr = PetscUseMethod(pep,"PEPJDGetRestart_C",(PEP,PetscReal*),(pep,keep));CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+PetscErrorCode PEPSetFromOptions_JD(PetscOptionItems *PetscOptionsObject,PEP pep)
+{
+  PetscErrorCode ierr;
+  PetscBool      flg;
+  PetscReal      r1;
+
+  PetscFunctionBegin;
+  ierr = PetscOptionsHead(PetscOptionsObject,"PEP JD Options");CHKERRQ(ierr);
+
+    ierr = PetscOptionsReal("-pep_jd_restart","Proportion of vectors kept after restart","PEPJDSetRestart",0.5,&r1,&flg);CHKERRQ(ierr);
+    if (flg) { ierr = PEPJDSetRestart(pep,r1);CHKERRQ(ierr); }
+
+  ierr = PetscOptionsTail();CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+PetscErrorCode PEPView_JD(PEP pep,PetscViewer viewer)
+{
+  PetscErrorCode ierr;
+  PEP_JD         *pjd = (PEP_JD*)pep->data;
+  PetscBool      isascii;
+
+  PetscFunctionBegin;
+  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
+  if (isascii) {
+    ierr = PetscViewerASCIIPrintf(viewer,"  %d%% of basis vectors kept after restart\n",(int)(100*pjd->keep));CHKERRQ(ierr);
+  }
+  PetscFunctionReturn(0);
+}
+
+PetscErrorCode PEPSetDefaultST_JD(PEP pep)
+{
+  PetscErrorCode ierr;
+  KSP            ksp;
+
+  PetscFunctionBegin;
+  if (!((PetscObject)pep->st)->type_name) {
+    ierr = STSetType(pep->st,STPRECOND);CHKERRQ(ierr);
+    ierr = STPrecondSetKSPHasMat(pep->st,PETSC_TRUE);CHKERRQ(ierr);
+  }
+  ierr = STSetTransform(pep->st,PETSC_FALSE);CHKERRQ(ierr);
+  ierr = STGetKSP(pep->st,&ksp);CHKERRQ(ierr);
+  if (!((PetscObject)ksp)->type_name) {
+    ierr = KSPSetType(ksp,KSPBCGSL);CHKERRQ(ierr);
+    ierr = KSPSetTolerances(ksp,1e-5,PETSC_DEFAULT,PETSC_DEFAULT,100);CHKERRQ(ierr);
+  }
+  PetscFunctionReturn(0);
+}
+
 PetscErrorCode PEPReset_JD(PEP pep)
 {
   PetscErrorCode ierr;
@@ -1064,8 +1173,6 @@ PetscErrorCode PEPReset_JD(PEP pep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPDestroy_JD"
 PetscErrorCode PEPDestroy_JD(PEP pep)
 {
   PetscErrorCode ierr;
@@ -1077,8 +1184,6 @@ PetscErrorCode PEPDestroy_JD(PEP pep)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPCreate_JD"
 PETSC_EXTERN PetscErrorCode PEPCreate_JD(PEP pep)
 {
   PEP_JD         *pjd;
@@ -1088,14 +1193,16 @@ PETSC_EXTERN PetscErrorCode PEPCreate_JD(PEP pep)
   ierr = PetscNewLog(pep,&pjd);CHKERRQ(ierr);
   pep->data = (void*)pjd;
 
-  pjd->keep = 0;
   pep->ops->solve          = PEPSolve_JD;
   pep->ops->setup          = PEPSetUp_JD;
   pep->ops->setfromoptions = PEPSetFromOptions_JD;
-  pep->ops->reset          = PEPReset_JD;
   pep->ops->destroy        = PEPDestroy_JD;
+  pep->ops->reset          = PEPReset_JD;
   pep->ops->view           = PEPView_JD;
+  pep->ops->setdefaultst   = PEPSetDefaultST_JD;
+
   ierr = PetscObjectComposeFunction((PetscObject)pep,"PEPJDSetRestart_C",PEPJDSetRestart_JD);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)pep,"PEPJDGetRestart_C",PEPJDGetRestart_JD);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
+
