@@ -1,3 +1,12 @@
+/*
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+   SLEPc - Scalable Library for Eigenvalue Problem Computations
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+
+   This file is part of SLEPc.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
+   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+*/
 /*
    Newton refinement for polynomial eigenproblems.
 
@@ -8,26 +17,8 @@
            435(3):514-536, 2011.
 
        [2] C. Campos and J.E. Roman, "Parallel iterative refinement in
-           polynomial eigenvalue problems", submitted, 2015.
-
-   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-   SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
-
-   This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
-   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+           polynomial eigenvalue problems", Numer. Linear Algebra Appl. 23(4):
+           730-745, 2016.
 */
 
 #include <slepc/private/pepimpl.h>
@@ -59,8 +50,6 @@ typedef struct {
   PetscInt     idx,*cols;
 } MatExplicitCtx;
 
-#undef __FUNCT__
-#define __FUNCT__ "MatFSMult"
 static PetscErrorCode MatFSMult(Mat M ,Vec x,Vec y)
 {
 #if defined(PETSC_MISSING_LAPACK_GETRS)
@@ -84,13 +73,12 @@ static PetscErrorCode MatFSMult(Mat M ,Vec x,Vec y)
   ierr = BVDotVec(ctx->M3,ctx->t,c);CHKERRQ(ierr);
   for (i=0;i<k;i++) c[i] = PetscConj(c[i]);
   PetscStackCallBLAS("LAPACKgetrs",LAPACKgetrs_("N",&k_,&one,ctx->M4,&k_,ctx->pM4,c,&k_,&info));
+  SlepcCheckLapackInfo("getrs",info);
   ierr = BVMultVec(ctx->M2,-1.0,1.0,y,c);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 #endif
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPEvaluateBasisforMatrix"
 /*
   Evaluates the first d elements of the polynomial basis
   on a given matrix H which is considered to be triangular
@@ -139,8 +127,6 @@ static PetscErrorCode PEPEvaluateBasisforMatrix(PEP pep,PetscInt nm,PetscInt k,P
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NRefSysSetup_shell"
 static PetscErrorCode NRefSysSetup_shell(PEP pep,PetscInt k,PetscScalar *fH,PetscScalar *S,PetscInt lds,PetscScalar *fh,PetscScalar h,FSubctx *ctx)
 {
 #if defined(PETSC_MISSING_LAPACK_GESV)
@@ -236,6 +222,7 @@ static PetscErrorCode NRefSysSetup_shell(PEP pep,PetscInt k,PetscScalar *fH,Pets
   ierr = VecGetArray(ctx->t,&v);CHKERRQ(ierr);
   for (i=0;i<nloc;i++) for (j=0;j<k;j++) T[j+i*k] = m3[i+j*nloc];
   PetscStackCallBLAS("LAPACKgesv",LAPACKgesv_(&k_,&nloc_,ctx->M4,&k_,ctx->pM4,T,&k_,&info));
+  SlepcCheckLapackInfo("gesv",info);
   for (i=0;i<nloc;i++) v[i] = BLASdot_(&k_,m2+i,&nloc_,T+i*k,&one);
   ierr = VecRestoreArray(ctx->t,&v);CHKERRQ(ierr);
   ierr = BVRestoreArrayRead(ctx->M2,&m2);CHKERRQ(ierr);
@@ -247,8 +234,6 @@ static PetscErrorCode NRefSysSetup_shell(PEP pep,PetscInt k,PetscScalar *fH,Pets
 #endif
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NRefSysSolve_shell"
 static PetscErrorCode NRefSysSolve_shell(KSP ksp,PetscInt nmat,Vec Rv,PetscScalar *Rh,PetscInt k,Vec dVi,PetscScalar *dHi)
 {
 #if defined(PETSC_MISSING_LAPACK_GETRS)
@@ -270,6 +255,7 @@ static PetscErrorCode NRefSysSolve_shell(KSP ksp,PetscInt nmat,Vec Rv,PetscScala
   ierr = PetscBLASIntCast(k,&k_);CHKERRQ(ierr);
   for (i=0;i<k;i++) t0[i] = Rh[i];
   PetscStackCallBLAS("LAPACKgetrs",LAPACKgetrs_("N",&k_,&one,ctx->M4,&k_,ctx->pM4,t0,&k_,&info));
+  SlepcCheckLapackInfo("getrs",info);
   ierr = BVMultVec(ctx->M2,-1.0,1.0,Rv,t0);CHKERRQ(ierr);
   ierr = KSPSolve(ksp,Rv,dVi);CHKERRQ(ierr);
   ierr = VecConjugate(dVi);CHKERRQ(ierr);
@@ -277,13 +263,12 @@ static PetscErrorCode NRefSysSolve_shell(KSP ksp,PetscInt nmat,Vec Rv,PetscScala
   ierr = VecConjugate(dVi);CHKERRQ(ierr);
   for (i=0;i<k;i++) dHi[i] = Rh[i]-PetscConj(dHi[i]);
   PetscStackCallBLAS("LAPACKgetrs",LAPACKgetrs_("N",&k_,&one,ctx->M4,&k_,ctx->pM4,dHi,&k_,&info));
+  SlepcCheckLapackInfo("getrs",info);
   ierr = PetscFree(t0);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 #endif
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NRefRightSide"
 /*
    Computes the residual P(H,V*S)*e_j for the polynomial
 */
@@ -373,8 +358,6 @@ static PetscErrorCode NRefRightSide(PetscInt nmat,PetscReal *pcf,Mat *A,PetscInt
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NRefSysSolve_mbe"
 static PetscErrorCode NRefSysSolve_mbe(PetscInt k,PetscInt sz,BV W,PetscScalar *w,BV Wt,PetscScalar *wt,PetscScalar *d,PetscScalar *dt,KSP ksp,BV T2,BV T3 ,PetscScalar *T4,PetscBool trans,Vec x1,PetscScalar *x2,Vec sol1,PetscScalar *sol2,Vec vw)
 {
   PetscErrorCode ierr;
@@ -436,8 +419,6 @@ static PetscErrorCode NRefSysSolve_mbe(PetscInt k,PetscInt sz,BV W,PetscScalar *
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NRefSysSetup_mbe"
 static PetscErrorCode NRefSysSetup_mbe(PEP pep,PetscInt k,KSP ksp,PetscScalar *fH,PetscScalar *S,PetscInt lds,PetscScalar *fh,PetscScalar h,BV V,MatExplicitCtx *matctx)
 {
   PetscErrorCode ierr;
@@ -459,7 +440,7 @@ static PetscErrorCode NRefSysSetup_mbe(PEP pep,PetscInt k,KSP ksp,PetscScalar *f
   if (flg) {
     ierr = PetscMalloc1(pep->nmat,&At);CHKERRQ(ierr);
     for (i=0;i<pep->nmat;i++) {
-      ierr = STGetTOperators(pep->st,i,&At[i]);CHKERRQ(ierr);
+      ierr = STGetMatrixTransformed(pep->st,i,&At[i]);CHKERRQ(ierr);
     }
   } else At = pep->A;
   if (matctx->subc) A = matctx->A;
@@ -569,8 +550,6 @@ static PetscErrorCode NRefSysSetup_mbe(PEP pep,PetscInt k,KSP ksp,PetscScalar *f
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NRefSysSetup_explicit"
 static PetscErrorCode NRefSysSetup_explicit(PEP pep,PetscInt k,KSP ksp,PetscScalar *fH,PetscScalar *S,PetscInt lds,PetscScalar *fh,PetscScalar h,BV V,MatExplicitCtx *matctx,BV W)
 {
   PetscErrorCode    ierr;
@@ -598,7 +577,7 @@ static PetscErrorCode NRefSysSetup_explicit(PEP pep,PetscInt k,KSP ksp,PetscScal
   if (flg) {
     ierr = PetscMalloc1(pep->nmat,&At);CHKERRQ(ierr);
     for (i=0;i<pep->nmat;i++) {
-      ierr = STGetTOperators(pep->st,i,&At[i]);CHKERRQ(ierr);
+      ierr = STGetMatrixTransformed(pep->st,i,&At[i]);CHKERRQ(ierr);
     }
   } else At = pep->A;
   if (matctx->subc) A = matctx->A;
@@ -716,8 +695,6 @@ static PetscErrorCode NRefSysSetup_explicit(PEP pep,PetscInt k,KSP ksp,PetscScal
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NRefSysSolve_explicit"
 static PetscErrorCode NRefSysSolve_explicit(PetscInt k,KSP ksp,Vec Rv,PetscScalar *Rh,Vec dVi,PetscScalar *dHi,MatExplicitCtx *matctx)
 {
   PetscErrorCode    ierr;
@@ -759,8 +736,6 @@ static PetscErrorCode NRefSysSolve_explicit(PetscInt k,KSP ksp,Vec Rv,PetscScala
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NRefSysIter"
 static PetscErrorCode NRefSysIter(PetscInt i,PEP pep,PetscInt k,KSP ksp,PetscScalar *fH,PetscScalar *S,PetscInt lds,PetscScalar *fh,PetscScalar *H,PetscInt ldh,Vec Rv,PetscScalar *Rh,BV V,Vec dVi,PetscScalar *dHi,MatExplicitCtx *matctx,BV W)
 {
   PetscErrorCode    ierr;
@@ -871,8 +846,6 @@ static PetscErrorCode NRefSysIter(PetscInt i,PEP pep,PetscInt k,KSP ksp,PetscSca
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPNRefForwardSubstitution"
 static PetscErrorCode PEPNRefForwardSubstitution(PEP pep,PetscInt k,PetscScalar *S,PetscInt lds,PetscScalar *H,PetscInt ldh,PetscScalar *fH,BV dV,PetscScalar *dVS,PetscInt *rds,PetscScalar *dH,PetscInt lddh,KSP ksp,MatExplicitCtx *matctx)
 {
   PetscErrorCode ierr;
@@ -925,7 +898,7 @@ static PetscErrorCode PEPNRefForwardSubstitution(PEP pep,PetscInt k,PetscScalar
   if (flg) {
     ierr = PetscMalloc1(pep->nmat,&At);CHKERRQ(ierr);
     for (i=0;i<pep->nmat;i++) {
-      ierr = STGetTOperators(pep->st,i,&At[i]);CHKERRQ(ierr);
+      ierr = STGetMatrixTransformed(pep->st,i,&At[i]);CHKERRQ(ierr);
     }
   } else At = pep->A;
 
@@ -962,49 +935,23 @@ static PetscErrorCode PEPNRefForwardSubstitution(PEP pep,PetscInt k,PetscScalar
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NRefOrthogStep"
-static PetscErrorCode NRefOrthogStep(PEP pep,PetscInt k,PetscScalar *H,PetscInt ldh,PetscScalar *fH,PetscScalar *S,PetscInt lds,PetscInt *prs)
+static PetscErrorCode NRefOrthogStep(PEP pep,PetscInt k,PetscScalar *H,PetscInt ldh,PetscScalar *fH,PetscScalar *S,PetscInt lds)
 {
-#if defined(PETSC_MISSING_LAPACK_GEQRF) || defined(PETSC_MISSING_LAPACK_ORGQR)
+#if defined(PETSC_MISSING_LAPACK_GEQRF)
   PetscFunctionBegin;
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"GEQRF/ORGQR - Lapack routine is unavailable");
+  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"GEQRF - Lapack routine is unavailable");
 #else
   PetscErrorCode ierr;
-  PetscInt       i,j,nmat=pep->nmat,deg=nmat-1,lda=nmat*k,rs=*prs,ldg;
-  PetscScalar    *T,*G,*tau,*array,sone=1.0,zero=0.0,*work;
-  PetscBLASInt   rs_,lds_,k_,ldh_,info,ldg_,lda_;
-  Mat            M0;
+  PetscInt       j,nmat=pep->nmat,deg=nmat-1,lda=nmat*k,ldg;
+  PetscScalar    *G,*tau,sone=1.0,zero=0.0,*work;
+  PetscBLASInt   lds_,k_,ldh_,info,ldg_,lda_;
 
   PetscFunctionBegin;
-  ierr = PetscMalloc4(rs*k,&T,k,&tau,k,&work,deg*k*k,&G);CHKERRQ(ierr);
+  ierr = PetscMalloc3(k,&tau,k,&work,deg*k*k,&G);CHKERRQ(ierr);
   ierr = PetscBLASIntCast(lds,&lds_);CHKERRQ(ierr);
   ierr = PetscBLASIntCast(lda,&lda_);CHKERRQ(ierr);
   ierr = PetscBLASIntCast(k,&k_);CHKERRQ(ierr);
-  if (rs>k) { /* Truncate S to have k columns*/
-    for (j=0;j<k;j++) {
-      ierr = PetscMemcpy(T+j*rs,S+j*lds,rs*sizeof(PetscScalar));CHKERRQ(ierr);
-    }
-    ierr = PetscBLASIntCast(rs,&rs_);CHKERRQ(ierr);
-    PetscStackCallBLAS("LAPACKgeqrf",LAPACKgeqrf_(&rs_,&k_,T,&rs_,tau,work,&k_,&info));
-    if (info) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"Error in Lapack xGEQRF %d",info);
-    /* Copy triangular matrix in S */
-    ierr = PetscMemzero(S,lds*k*sizeof(PetscScalar));CHKERRQ(ierr);
-    for (j=0;j<k;j++) for (i=0;i<=j;i++) S[j*lds+i] = T[j*rs+i];
-    PetscStackCallBLAS("LAPACKungqr",LAPACKungqr_(&rs_,&k_,&k_,T,&rs_,tau,work,&k_,&info));
-    if (info) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"Error in Lapack xORGQR %d",info);
-    ierr = MatCreateSeqDense(PETSC_COMM_SELF,rs,k,NULL,&M0);CHKERRQ(ierr);
-    ierr = MatDenseGetArray(M0,&array);CHKERRQ(ierr);
-    for (j=0;j<k;j++) {
-      ierr = PetscMemcpy(array+j*rs,T+j*rs,rs*sizeof(PetscScalar));CHKERRQ(ierr);
-    }
-    ierr = MatDenseRestoreArray(M0,&array);CHKERRQ(ierr);
-    ierr = BVSetActiveColumns(pep->V,0,rs);CHKERRQ(ierr);
-    ierr = BVMultInPlace(pep->V,M0,0,k);CHKERRQ(ierr);
-    ierr = BVSetActiveColumns(pep->V,0,k);CHKERRQ(ierr);
-    ierr = MatDestroy(&M0);CHKERRQ(ierr);
-    *prs = rs = k;
-  }
+
   /* Form auxiliary matrix for the orthogonalization step */
   ldg = deg*k;
   ierr = PEPEvaluateBasisforMatrix(pep,nmat,k,H,ldh,fH);CHKERRQ(ierr);
@@ -1015,19 +962,17 @@ static PetscErrorCode NRefOrthogStep(PEP pep,PetscInt k,PetscScalar *H,PetscInt
   }
   /* Orthogonalize and update S */
   PetscStackCallBLAS("LAPACKgeqrf",LAPACKgeqrf_(&ldg_,&k_,G,&ldg_,tau,work,&k_,&info));
-  if (info) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"Error in Lapack xGEQRF %d",info);
+  SlepcCheckLapackInfo("geqrf",info);
   PetscStackCallBLAS("BLAStrsm",BLAStrsm_("R","U","N","N",&k_,&k_,&sone,G,&ldg_,S,&lds_));
 
   /* Update H */
   PetscStackCallBLAS("BLAStrmm",BLAStrmm_("L","U","N","N",&k_,&k_,&sone,G,&ldg_,H,&ldh_));
   PetscStackCallBLAS("BLAStrsm",BLAStrsm_("R","U","N","N",&k_,&k_,&sone,G,&ldg_,H,&ldh_));
-  ierr = PetscFree4(T,tau,work,G);CHKERRQ(ierr);
+  ierr = PetscFree3(tau,work,G);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 #endif
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPNRefUpdateInvPair"
 static PetscErrorCode PEPNRefUpdateInvPair(PEP pep,PetscInt k,PetscScalar *H,PetscInt ldh,PetscScalar *fH,PetscScalar *dH,PetscScalar *S,PetscInt lds,BV dV,PetscScalar *dVS,PetscInt rds)
 {
 #if defined(PETSC_MISSING_LAPACK_GEQRF) || defined(PETSC_MISSING_LAPACK_ORGQR)
@@ -1058,14 +1003,14 @@ static PetscErrorCode PEPNRefUpdateInvPair(PEP pep,PetscInt k,PetscScalar *H,Pet
     for (i=k;i<2*k;i++) dVS[i+j*2*k] = -dVS[i+j*2*k];
   }
   PetscStackCallBLAS("LAPACKgeqrf",LAPACKgeqrf_(&kdrs_,&k_,dVS,&k2_,tau,work,&k_,&info));
-  if (info) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"Error in Lapack xGEQRF %d",info);
+  SlepcCheckLapackInfo("geqrf",info);
   /* Copy triangular matrix in S */
   for (j=0;j<k;j++) {
     for (i=0;i<=j;i++) S[i+j*lds] = dVS[i+j*2*k];
     for (i=j+1;i<k;i++) S[i+j*lds] = 0.0;
   }
   PetscStackCallBLAS("LAPACKungqr",LAPACKungqr_(&k2_,&k_,&k_,dVS,&k2_,tau,work,&k_,&info));
-  if (info) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"Error in Lapack xORGQR %d",info);
+  SlepcCheckLapackInfo("ungqr",info);
   ierr = MatCreateSeqDense(PETSC_COMM_SELF,k,k,NULL,&M0);CHKERRQ(ierr);
   ierr = MatDenseGetArray(M0,&array);CHKERRQ(ierr);
   for (j=0;j<k;j++) {
@@ -1083,14 +1028,12 @@ static PetscErrorCode PEPNRefUpdateInvPair(PEP pep,PetscInt k,PetscScalar *H,Pet
     ierr = BVMult(pep->V,1.0,1.0,dV,NULL);CHKERRQ(ierr);
   }
   ierr = MatDestroy(&M0);CHKERRQ(ierr);
-  ierr = NRefOrthogStep(pep,k,H,ldh,fH,S,lds,&k);CHKERRQ(ierr);
+  ierr = NRefOrthogStep(pep,k,H,ldh,fH,S,lds);CHKERRQ(ierr);
   ierr = PetscFree2(tau,work);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 #endif
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPNRefSetUp"
 static PetscErrorCode PEPNRefSetUp(PEP pep,PetscInt k,PetscScalar *H,PetscInt ldh,MatExplicitCtx *matctx,PetscBool ini)
 {
   PetscErrorCode    ierr;
@@ -1114,7 +1057,7 @@ static PetscErrorCode PEPNRefSetUp(PEP pep,PetscInt k,PetscScalar *H,PetscInt ld
   if (flg) {
     ierr = PetscMalloc1(pep->nmat,&At);CHKERRQ(ierr);
     for (i=0;i<pep->nmat;i++) {
-      ierr = STGetTOperators(pep->st,i,&At[i]);CHKERRQ(ierr);
+      ierr = STGetMatrixTransformed(pep->st,i,&At[i]);CHKERRQ(ierr);
     }
   } else At = pep->A;
   switch (pep->scheme) {
@@ -1151,7 +1094,7 @@ static PetscErrorCode PEPNRefSetUp(PEP pep,PetscInt k,PetscScalar *H,PetscInt ld
       ierr = MatCreateDense(comm,m1-n1,m0_-n0_,PETSC_DECIDE,PETSC_DECIDE,NULL,&C);CHKERRQ(ierr);
       ierr = MatAssemblyBegin(C,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
       ierr = MatAssemblyEnd(C,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
-      ierr = SlepcMatTile(1.0,E[0],1.0,B,1.0,C,1.0,E[1],&M);CHKERRQ(ierr);
+      ierr = MatCreateTile(1.0,E[0],1.0,B,1.0,C,1.0,E[1],&M);CHKERRQ(ierr);
       ierr = MatDestroy(&B);CHKERRQ(ierr);
       ierr = MatDestroy(&C);CHKERRQ(ierr);
       matctx->compM1 = PETSC_TRUE;
@@ -1252,10 +1195,10 @@ static PetscErrorCode PEPNRefSetUp(PEP pep,PetscInt k,PetscScalar *H,PetscInt ld
       ierr = PetscMalloc1(1,&ctx);CHKERRQ(ierr);
       ierr = STGetMatStructure(pep->st,&str);CHKERRQ(ierr);
       /* Create a shell matrix to solve the linear system */
-      ctx->A = At;
       ctx->V = pep->V;
       ctx->k = k; ctx->nmat = nmat;
-      ierr = PetscMalloc4(k*k,&ctx->M4,k,&ctx->pM4,2*k*k,&ctx->work,nmat,&ctx->fih);CHKERRQ(ierr);
+      ierr = PetscMalloc5(nmat,&ctx->A,k*k,&ctx->M4,k,&ctx->pM4,2*k*k,&ctx->work,nmat,&ctx->fih);CHKERRQ(ierr);
+      for (i=0;i<nmat;i++) ctx->A[i] = At[i];
       ierr = PetscMemzero(ctx->M4,k*k*sizeof(PetscScalar));CHKERRQ(ierr);
       ierr = MatCreateShell(comm,PETSC_DECIDE,PETSC_DECIDE,m0,n0,ctx,&M);CHKERRQ(ierr);
       ierr = MatShellSetOperation(M,MATOP_MULT,(void(*)(void))MatFSMult);CHKERRQ(ierr);
@@ -1307,8 +1250,6 @@ static PetscErrorCode PEPNRefSetUp(PEP pep,PetscInt k,PetscScalar *H,PetscInt ld
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NRefSubcommSetup"
 static PetscErrorCode NRefSubcommSetup(PEP pep,PetscInt k,MatExplicitCtx *matctx,PetscInt nsubc)
 {
   PetscErrorCode    ierr;
@@ -1325,7 +1266,7 @@ static PetscErrorCode NRefSubcommSetup(PEP pep,PetscInt k,MatExplicitCtx *matctx
   if (flg) {
     ierr = PetscMalloc1(pep->nmat,&A);CHKERRQ(ierr);
     for (i=0;i<pep->nmat;i++) {
-      ierr = STGetTOperators(pep->st,i,&A[i]);CHKERRQ(ierr);
+      ierr = STGetMatrixTransformed(pep->st,i,&A[i]);CHKERRQ(ierr);
     }
   } else A = pep->A;
 
@@ -1398,8 +1339,6 @@ static PetscErrorCode NRefSubcommSetup(PEP pep,PetscInt k,MatExplicitCtx *matctx
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "NRefSubcommDestroy"
 static PetscErrorCode NRefSubcommDestroy(PEP pep,MatExplicitCtx *matctx)
 {
   PetscErrorCode ierr;
@@ -1430,9 +1369,7 @@ static PetscErrorCode NRefSubcommDestroy(PEP pep,MatExplicitCtx *matctx)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PEPNewtonRefinement_TOAR"
-PetscErrorCode PEPNewtonRefinement_TOAR(PEP pep,PetscScalar sigma,PetscInt *maxits,PetscReal *tol,PetscInt k,PetscScalar *S,PetscInt lds,PetscInt *prs)
+PetscErrorCode PEPNewtonRefinement_TOAR(PEP pep,PetscScalar sigma,PetscInt *maxits,PetscReal *tol,PetscInt k,PetscScalar *S,PetscInt lds)
 {
 #if defined(PETSC_MISSING_LAPACK_GETRF) || defined(PETSC_MISSING_LAPACK_GETRI)
   PetscFunctionBegin;
@@ -1468,7 +1405,9 @@ PetscErrorCode PEPNewtonRefinement_TOAR(PEP pep,PetscScalar sigma,PetscInt *maxi
       ierr = DSGetArray(pep->ds,DS_MAT_A,&H);CHKERRQ(ierr);
       ierr = PetscMalloc1(k,&p);CHKERRQ(ierr);
       PetscStackCallBLAS("LAPACKgetrf",LAPACKgetrf_(&k_,&k_,H,&ld_,p,&info));
+      SlepcCheckLapackInfo("getrf",info);
       PetscStackCallBLAS("LAPACKgetri",LAPACKgetri_(&k_,H,&ld_,p,work,&k_,&info));
+      SlepcCheckLapackInfo("getri",info);
       ierr = DSRestoreArray(pep->ds,DS_MAT_A,&H);CHKERRQ(ierr);
       pep->ops->backtransform = NULL;
     }
@@ -1502,7 +1441,7 @@ PetscErrorCode PEPNewtonRefinement_TOAR(PEP pep,PetscScalar sigma,PetscInt *maxi
   }
   ierr = DSGetArray(pep->ds,DS_MAT_A,&H);CHKERRQ(ierr);
 
-  ierr = NRefOrthogStep(pep,k,H,ldh,fH,S,lds,prs);CHKERRQ(ierr);
+  ierr = NRefOrthogStep(pep,k,H,ldh,fH,S,lds);CHKERRQ(ierr);
   /* check if H is in Schur form */
   for (i=0;i<k-1;i++) {
     if (H[i+1+i*ldh]!=0.0) {
@@ -1577,7 +1516,7 @@ PetscErrorCode PEPNewtonRefinement_TOAR(PEP pep,PetscScalar sigma,PetscInt *maxi
   case PEP_REFINE_SCHEME_SCHUR:
     ierr = KSPGetOperators(pep->refineksp,&M,&P);CHKERRQ(ierr);
     ierr = MatShellGetContext(M,&ctx);CHKERRQ(ierr);
-    ierr = PetscFree4(ctx->M4,ctx->pM4,ctx->work,ctx->fih);CHKERRQ(ierr);
+    ierr = PetscFree5(ctx->A,ctx->M4,ctx->pM4,ctx->work,ctx->fih);CHKERRQ(ierr);
     ierr = MatDestroy(&ctx->M1);CHKERRQ(ierr);
     ierr = BVDestroy(&ctx->M2);CHKERRQ(ierr);
     ierr = BVDestroy(&ctx->M3);CHKERRQ(ierr);
