@@ -1,24 +1,13 @@
 /*
-
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
+
 #include <slepc/private/dsimpl.h>
 #include <slepcblaslapack.h>
 
@@ -35,8 +24,6 @@ struct HRtr
   PetscInt    type;
 };
 
-#undef __FUNCT__
-#define __FUNCT__ "HRGen"
 /*
   Generates a hyperbolic rotation
     if x1*x1 - x2*x2 != 0
@@ -87,8 +74,6 @@ static PetscErrorCode HRGen(PetscReal x1,PetscReal x2,PetscInt *type,PetscReal *
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "HRApply"
 /*
                                 |c  s|
   Applies an hyperbolic rotator |s  c|
@@ -119,8 +104,6 @@ static PetscErrorCode HRApply(PetscInt n,PetscScalar *x1,PetscInt inc1,PetscScal
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TridiagDiag_HHR"
 /*
   Reduction to tridiagonal-diagonal form (see F. Tisseur, SIMAX 26(1), 2004).
 
@@ -256,8 +239,7 @@ static PetscErrorCode TridiagDiag_HHR(PetscInt n,PetscScalar *A,PetscInt lda,Pet
         if (cond > 1.0/(10*PETSC_SQRT_MACHINE_EPSILON)) {
           breakdown = PETSC_TRUE;
           k++;
-          if (k==n || flip)
-            SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Breakdown in construction of hyperbolic transformation");
+          if (k==n || flip) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Breakdown in construction of hyperbolic transformation");
           break;
         }
         A[ni-n0+j*lda] = r; A[n-n1+j*lda] = 0.0;
@@ -306,8 +288,6 @@ static PetscErrorCode TridiagDiag_HHR(PetscInt n,PetscScalar *A,PetscInt lda,Pet
 #endif
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MadeHRtr"
 static PetscErrorCode MadeHRtr(PetscInt sz,PetscInt n,PetscInt idx0,PetscInt n0,PetscInt idx1,PetscInt n1,struct HRtr *tr1,struct HRtr *tr2,PetscReal *ncond,PetscScalar *work)
 {
 #if defined(SLEPC_MISSING_LAPACK_LARFG) || defined(SLEPC_MISSING_LAPACK_LARF)
@@ -335,7 +315,7 @@ static PetscErrorCode MadeHRtr(PetscInt sz,PetscInt n,PetscInt idx0,PetscInt n0,
     PetscStackCallBLAS("LAPACKlarfg",LAPACKlarfg_(&n1_,x+tr1->idx[1],x+tr1->idx[1]+1,&inc,tr1->tau+1));
   }
   if (tr1->idx[0]<tr1->idx[1]) {
-    ierr = HRGen(PetscRealPart(x[tr1->idx[0]]),PetscRealPart(x[tr1->idx[1]]),&(tr1->type),&(tr1->cs),&(tr1->sn),&(tr1->alpha),ncond);CHKERRQ(ierr);  
+    ierr = HRGen(PetscRealPart(x[tr1->idx[0]]),PetscRealPart(x[tr1->idx[1]]),&(tr1->type),&(tr1->cs),&(tr1->sn),&(tr1->alpha),ncond);CHKERRQ(ierr);
   } else {
     tr1->alpha = PetscRealPart(x[tr1->idx[0]]);
     *ncond = 1.0;
@@ -377,7 +357,7 @@ static PetscErrorCode MadeHRtr(PetscInt sz,PetscInt n,PetscInt idx0,PetscInt n0,
       PetscStackCallBLAS("LAPACKlarfg",LAPACKlarfg_(&n1_,y+tr2->idx[1],y+tr2->idx[1]+1,&inc,tr2->tau+1));
     }
     if (tr2->idx[0]<tr2->idx[1]) {
-      ierr = HRGen(PetscRealPart(y[tr2->idx[0]]),PetscRealPart(y[tr2->idx[1]]),&(tr2->type),&(tr2->cs),&(tr2->sn),&(tr2->alpha),&ncond2);CHKERRQ(ierr);  
+      ierr = HRGen(PetscRealPart(y[tr2->idx[0]]),PetscRealPart(y[tr2->idx[1]]),&(tr2->type),&(tr2->cs),&(tr2->sn),&(tr2->alpha),&ncond2);CHKERRQ(ierr);
     } else {
     tr2->alpha = PetscRealPart(y[tr2->idx[0]]);
     ncond2 = 1.0;
@@ -388,8 +368,6 @@ static PetscErrorCode MadeHRtr(PetscInt sz,PetscInt n,PetscInt idx0,PetscInt n0,
 #endif
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TryHRIt"
 /*
   Auxiliary function to try perform one iteration of hr routine,
   checking condition number. If it is < tolD, apply the
@@ -425,11 +403,9 @@ static PetscErrorCode TryHRIt(PetscInt n,PetscInt j,PetscInt sz,PetscScalar *H,P
     /* Hyperbolic transformation to make zeros in x */
     ierr = MadeHRtr(sz,n,*idx0,*n0,*idx1,*n1,&tr1_t,NULL,&ncond,work+nwu);CHKERRQ(ierr);
     /* Check condition number to single column*/
-    if (ncond>tolD) {
-      *ok = PETSC_FALSE;
-    }
+    if (ncond>tolD) *ok = PETSC_FALSE;
     tr1 = &tr1_t;
-    tr2 = &tr2_t;    
+    tr2 = &tr2_t;
   } else {
     y = work+nwu;
     nwu += n;
@@ -461,7 +437,7 @@ static PetscErrorCode TryHRIt(PetscInt n,PetscInt j,PetscInt sz,PetscScalar *H,P
     if (cond && *cond<ncond) *cond = ncond;
     x = tr1->data;
     ierr = PetscBLASIntCast(tr1->n[0],&n0_);CHKERRQ(ierr);
-    ierr = PetscBLASIntCast(tr1->n[1],&n1_);CHKERRQ(ierr); 
+    ierr = PetscBLASIntCast(tr1->n[1],&n1_);CHKERRQ(ierr);
     ierr = PetscBLASIntCast(n-j-sz,&mr);CHKERRQ(ierr);
     if (tr1->n[0] > 1 && PetscAbsScalar(tr1->tau[0])!=0.0) {
       x[tr1->idx[0]] = 1.0;
@@ -483,14 +459,14 @@ static PetscErrorCode TryHRIt(PetscInt n,PetscInt j,PetscInt sz,PetscScalar *H,P
         s[tr1->idx[1]] = -s[tr1->idx[1]];
       }
     }
-    for (i=0;i<tr1->idx[0];i++) *(R+j*ldr+i) = x[i];  
+    for (i=0;i<tr1->idx[0];i++) *(R+j*ldr+i) = x[i];
     for (i=tr1->idx[0]+1;i<n;i++) *(R+j*ldr+i) = 0.0;
     *(R+j*ldr+tr1->idx[0]) = tr1->alpha;
     if (sz==2) {
       y = tr2->data;
       /* Second column */
       ierr = PetscBLASIntCast(tr2->n[0],&n0_);CHKERRQ(ierr);
-      ierr = PetscBLASIntCast(tr2->n[1],&n1_);CHKERRQ(ierr); 
+      ierr = PetscBLASIntCast(tr2->n[1],&n1_);CHKERRQ(ierr);
       ierr = PetscBLASIntCast(n-j-sz,&mr);CHKERRQ(ierr);
       ierr = PetscBLASIntCast(n-tr2->idx[0],&mh);CHKERRQ(ierr);
       if (tr2->n[0] > 1 && PetscAbsScalar(tr2->tau[0])!=0.0) {
@@ -531,7 +507,7 @@ static PetscErrorCode TryHRIt(PetscInt n,PetscInt j,PetscInt sz,PetscScalar *H,P
       *idx1 = tr1->idx[1];
       if (tr1->idx[0]<tr1->idx[1] && tr1->type==2) {
         (*idx1)++; (*n1)--; (*n0)++;
-      }      
+      }
     }
     if (*n0>0) {
       (*n0)--; (*idx0)++;
@@ -544,8 +520,6 @@ static PetscErrorCode TryHRIt(PetscInt n,PetscInt j,PetscInt sz,PetscScalar *H,P
 #endif
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PseudoOrthog_HR"
 /*
   compute V = HR whit H s-orthogonal and R upper triangular
 */
@@ -607,7 +581,7 @@ static PetscErrorCode PseudoOrthog_HR(PetscInt *nv,PetscScalar *V,PetscInt ldv,P
           for (i=j;i<n-1;i++) cmplxEig[i] = cmplxEig[i+1];
           cmplxEig[n-1] = t1;
           ierr = PetscMemcpy(col1,R+j*ldr,n*sizeof(PetscScalar));CHKERRQ(ierr);
-          for (i=j;i<n-1;i++) { 
+          for (i=j;i<n-1;i++) {
             ierr = PetscMemcpy(R+i*ldr,R+(i+1)*ldr,n*sizeof(PetscScalar));CHKERRQ(ierr);
           }
           ierr = PetscMemcpy(R+(n-1)*ldr,col1,n*sizeof(PetscScalar));CHKERRQ(ierr);
@@ -643,8 +617,6 @@ static PetscErrorCode PseudoOrthog_HR(PetscInt *nv,PetscScalar *V,PetscInt ldv,P
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSGHIEPOrthogEigenv"
 PetscErrorCode DSGHIEPOrthogEigenv(DS ds,DSMatType mat,PetscScalar *wr,PetscScalar *wi,PetscBool accum)
 {
   PetscErrorCode ierr;
@@ -688,11 +660,11 @@ PetscErrorCode DSGHIEPOrthogEigenv(DS ds,DSMatType mat,PetscScalar *wr,PetscScal
     } else cmplxEig[i] = 0;
   }
   nv = n;
-  
+
   /* Perform HR decomposition */
   /* Hyperbolic rotators */
   ierr = PseudoOrthog_HR(&nv,X+off,ld,s+l,R,ldr,perm,cmplxEig,NULL,ds->work+nwus);CHKERRQ(ierr);
-  /* Sort wr,wi perm */ 
+  /* Sort wr,wi perm */
   ts = ds->work+nwus;
   ierr = PetscMemcpy(ts,wr+l,n*sizeof(PetscScalar));CHKERRQ(ierr);
   for (i=0;i<n;i++) wr[i+l] = ts[perm[i]];
@@ -737,12 +709,10 @@ PetscErrorCode DSGHIEPOrthogEigenv(DS ds,DSMatType mat,PetscScalar *wr,PetscScal
     for (i=0;i<n;i++) { ierr = PetscMemcpy(ds->mat[DS_MAT_Q]+off+i*ld,X+off+i*ld,n*sizeof(PetscScalar));CHKERRQ(ierr); }
   }
   ds->t = nv+l;
-  if (!ds->compact) { ierr = DSSwitchFormat_GHIEP(ds,PETSC_FALSE);CHKERRQ(ierr); }  
+  if (!ds->compact) { ierr = DSSwitchFormat_GHIEP(ds,PETSC_FALSE);CHKERRQ(ierr); }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSIntermediate_GHIEP"
 /*
    Reduce to tridiagonal-diagonal pair by means of TridiagDiag_HHR.
 */
