@@ -1,29 +1,16 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2016, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
-
-   SLEPc is free software: you can redistribute it and/or modify it under  the
-   terms of version 3 of the GNU Lesser General Public License as published by
-   the Free Software Foundation.
-
-   SLEPc  is  distributed in the hope that it will be useful, but WITHOUT  ANY
-   WARRANTY;  without even the implied warranty of MERCHANTABILITY or  FITNESS
-   FOR  A  PARTICULAR PURPOSE. See the GNU Lesser General Public  License  for
-   more details.
-
-   You  should have received a copy of the GNU Lesser General  Public  License
-   along with SLEPc. If not, see <http://www.gnu.org/licenses/>.
+   SLEPc is distributed under a 2-clause BSD license (see LICENSE).
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
 
 #include <slepc/private/dsimpl.h>
 #include <slepcblaslapack.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "DSAllocate_NHEP"
 PetscErrorCode DSAllocate_NHEP(DS ds,PetscInt ld)
 {
   PetscErrorCode ierr;
@@ -37,8 +24,6 @@ PetscErrorCode DSAllocate_NHEP(DS ds,PetscInt ld)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSView_NHEP"
 PetscErrorCode DSView_NHEP(DS ds,PetscViewer viewer)
 {
   PetscErrorCode ierr;
@@ -57,8 +42,6 @@ PetscErrorCode DSView_NHEP(DS ds,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSVectors_NHEP_Refined_Some"
 static PetscErrorCode DSVectors_NHEP_Refined_Some(DS ds,PetscInt *k,PetscReal *rnorm,PetscBool left)
 {
 #if defined(PETSC_MISSING_LAPACK_GESVD)
@@ -77,7 +60,7 @@ static PetscErrorCode DSVectors_NHEP_Refined_Some(DS ds,PetscInt *k,PetscReal *r
   PetscScalar    *W;
 
   PetscFunctionBegin;
-  if (left) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Not implemented for left vectors");
+  if (left) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_SUP,"Not implemented for left vectors");
   ierr = PetscBLASIntCast(ds->n,&n);CHKERRQ(ierr);
   ierr = PetscBLASIntCast(ds->ld,&ld);CHKERRQ(ierr);
   n1 = n+1;
@@ -102,7 +85,7 @@ static PetscErrorCode DSVectors_NHEP_Refined_Some(DS ds,PetscInt *k,PetscReal *r
 #else
   PetscStackCallBLAS("LAPACKgesvd",LAPACKgesvd_("N","O",&n1,&n,W,&ld,sigma,&sdummy,&ld,&sdummy,&ld,ds->work,&lwork,ds->rwork,&info));
 #endif
-  if (info) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"Error in Lapack xGESVD %d",info);
+  SlepcCheckLapackInfo("gesvd",info);
 
   /* the smallest singular value is the new error estimate */
   if (rnorm) *rnorm = sigma[n-1];
@@ -114,8 +97,6 @@ static PetscErrorCode DSVectors_NHEP_Refined_Some(DS ds,PetscInt *k,PetscReal *r
 #endif
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSVectors_NHEP_Refined_All"
 static PetscErrorCode DSVectors_NHEP_Refined_All(DS ds,PetscBool left)
 {
   PetscErrorCode ierr;
@@ -128,8 +109,6 @@ static PetscErrorCode DSVectors_NHEP_Refined_All(DS ds,PetscBool left)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSVectors_NHEP_Eigen_Some"
 static PetscErrorCode DSVectors_NHEP_Eigen_Some(DS ds,PetscInt *k,PetscReal *rnorm,PetscBool left)
 {
 #if defined(SLEPC_MISSING_LAPACK_TREVC)
@@ -167,7 +146,7 @@ static PetscErrorCode DSVectors_NHEP_Eigen_Some(DS ds,PetscInt *k,PetscReal *rno
   ierr = DSAllocateWork_Private(ds,2*ld,ld,0);CHKERRQ(ierr);
   PetscStackCallBLAS("LAPACKtrevc",LAPACKtrevc_(left?"L":"R","S",select,&n,A,&ld,Y,&ld,Y,&ld,&mm,&mout,ds->work,ds->rwork,&info));
 #endif
-  if (info) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"Error in Lapack xTREVC %d",info);
+  SlepcCheckLapackInfo("trevc",info);
   if (mout != mm) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Inconsistent arguments");
 
   /* accumulate and normalize eigenvectors */
@@ -201,8 +180,6 @@ static PetscErrorCode DSVectors_NHEP_Eigen_Some(DS ds,PetscInt *k,PetscReal *rno
 #endif
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSVectors_NHEP_Eigen_All"
 static PetscErrorCode DSVectors_NHEP_Eigen_All(DS ds,PetscBool left)
 {
 #if defined(SLEPC_MISSING_LAPACK_TREVC)
@@ -242,7 +219,7 @@ static PetscErrorCode DSVectors_NHEP_Eigen_All(DS ds,PetscBool left)
   ierr = DSAllocateWork_Private(ds,2*ld,ld,0);CHKERRQ(ierr);
   PetscStackCallBLAS("LAPACKtrevc",LAPACKtrevc_(side,back,NULL,&n,A,&ld,Y,&ld,X,&ld,&n,&mout,ds->work,ds->rwork,&info));
 #endif
-  if (info) SETERRQ1(PetscObjectComm((PetscObject)ds),PETSC_ERR_LIB,"Error in Lapack xTREVC %i",info);
+  SlepcCheckLapackInfo("trevc",info);
 
   /* normalize eigenvectors */
   for (i=0;i<n;i++) {
@@ -265,8 +242,6 @@ static PetscErrorCode DSVectors_NHEP_Eigen_All(DS ds,PetscBool left)
 #endif
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSVectors_NHEP"
 PetscErrorCode DSVectors_NHEP(DS ds,DSMatType mat,PetscInt *j,PetscReal *rnorm)
 {
   PetscErrorCode ierr;
@@ -275,7 +250,7 @@ PetscErrorCode DSVectors_NHEP(DS ds,DSMatType mat,PetscInt *j,PetscReal *rnorm)
   switch (mat) {
     case DS_MAT_X:
       if (ds->refined) {
-        if (!ds->extrarow) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Refined vectors require activating the extra row");
+        if (!ds->extrarow) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_SUP,"Refined vectors require activating the extra row");
         if (j) {
           ierr = DSVectors_NHEP_Refined_Some(ds,j,rnorm,PETSC_FALSE);CHKERRQ(ierr);
         } else {
@@ -290,7 +265,7 @@ PetscErrorCode DSVectors_NHEP(DS ds,DSMatType mat,PetscInt *j,PetscReal *rnorm)
       }
       break;
     case DS_MAT_Y:
-      if (ds->refined) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Not implemented yet");
+      if (ds->refined) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_SUP,"Not implemented yet");
       if (j) {
         ierr = DSVectors_NHEP_Eigen_Some(ds,j,rnorm,PETSC_TRUE);CHKERRQ(ierr);
       } else {
@@ -299,7 +274,7 @@ PetscErrorCode DSVectors_NHEP(DS ds,DSMatType mat,PetscInt *j,PetscReal *rnorm)
       break;
     case DS_MAT_U:
     case DS_MAT_VT:
-      SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Not implemented yet");
+      SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_SUP,"Not implemented yet");
       break;
     default:
       SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_OUTOFRANGE,"Invalid mat parameter");
@@ -310,65 +285,6 @@ PetscErrorCode DSVectors_NHEP(DS ds,DSMatType mat,PetscInt *j,PetscReal *rnorm)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSNormalize_NHEP"
-PetscErrorCode DSNormalize_NHEP(DS ds,DSMatType mat,PetscInt col)
-{
-  PetscErrorCode ierr;
-  PetscInt       i,i0,i1;
-  PetscBLASInt   ld,n,one = 1;
-  PetscScalar    *A = ds->mat[DS_MAT_A],norm,*x;
-#if !defined(PETSC_USE_COMPLEX)
-  PetscScalar    norm0;
-#endif
-
-  PetscFunctionBegin;
-  switch (mat) {
-    case DS_MAT_X:
-    case DS_MAT_Y:
-    case DS_MAT_Q:
-      /* Supported matrices */
-      break;
-    case DS_MAT_U:
-    case DS_MAT_VT:
-      SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Not implemented yet");
-      break;
-    default:
-      SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_OUTOFRANGE,"Invalid mat parameter");
-  }
-
-  ierr = PetscBLASIntCast(ds->n,&n);CHKERRQ(ierr);
-  ierr = PetscBLASIntCast(ds->ld,&ld);CHKERRQ(ierr);
-  ierr = DSGetArray(ds,mat,&x);CHKERRQ(ierr);
-  if (col < 0) {
-    i0 = 0; i1 = ds->n;
-  } else if (col>0 && A[ds->ld*(col-1)+col] != 0.0) {
-    i0 = col-1; i1 = col+1;
-  } else {
-    i0 = col; i1 = col+1;
-  }
-  for (i=i0;i<i1;i++) {
-#if !defined(PETSC_USE_COMPLEX)
-    if (i<n-1 && A[ds->ld*i+i+1] != 0.0) {
-      norm = BLASnrm2_(&n,&x[ld*i],&one);
-      norm0 = BLASnrm2_(&n,&x[ld*(i+1)],&one);
-      norm = 1.0/SlepcAbsEigenvalue(norm,norm0);
-      PetscStackCallBLAS("BLASscal",BLASscal_(&n,&norm,&x[ld*i],&one));
-      PetscStackCallBLAS("BLASscal",BLASscal_(&n,&norm,&x[ld*(i+1)],&one));
-      i++;
-    } else
-#endif
-    {
-      norm = BLASnrm2_(&n,&x[ld*i],&one);
-      norm = 1.0/norm;
-      PetscStackCallBLAS("BLASscal",BLASscal_(&n,&norm,&x[ld*i],&one));
-    }
-  }
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "DSSort_NHEP_Arbitrary"
 static PetscErrorCode DSSort_NHEP_Arbitrary(DS ds,PetscScalar *wr,PetscScalar *wi,PetscScalar *rr,PetscScalar *ri,PetscInt *k)
 {
 #if defined(PETSC_MISSING_LAPACK_TRSEN)
@@ -384,7 +300,7 @@ static PetscErrorCode DSSort_NHEP_Arbitrary(DS ds,PetscScalar *wr,PetscScalar *w
 #endif
 
   PetscFunctionBegin;
-  if (!k) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Must supply argument k");
+  if (!k) SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_WRONG,"Must supply argument k");
   ierr = PetscBLASIntCast(ds->n,&n);CHKERRQ(ierr);
   ierr = PetscBLASIntCast(ds->ld,&ld);CHKERRQ(ierr);
 #if !defined(PETSC_USE_COMPLEX)
@@ -411,14 +327,12 @@ static PetscErrorCode DSSort_NHEP_Arbitrary(DS ds,PetscScalar *wr,PetscScalar *w
 #else
   PetscStackCallBLAS("LAPACKtrsen",LAPACKtrsen_("N","V",selection,&n,T,&ld,Q,&ld,wr,&mout,NULL,NULL,work,&lwork,&info));
 #endif
-  if (info) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"Error in Lapack xTRSEN %d",info);
+  SlepcCheckLapackInfo("trsen",info);
   *k = mout;
   PetscFunctionReturn(0);
 #endif
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSSort_NHEP_Total"
 static PetscErrorCode DSSort_NHEP_Total(DS ds,PetscScalar *wr,PetscScalar *wi)
 {
 #if defined(SLEPC_MISSING_LAPACK_TREXC)
@@ -480,7 +394,7 @@ static PetscErrorCode DSSort_NHEP_Total(DS ds,PetscScalar *wr,PetscScalar *wi)
 #else
       PetscStackCallBLAS("LAPACKtrexc",LAPACKtrexc_("V",&n,T,&ld,Q,&ld,&ifst,&ilst,&info));
 #endif
-      if (info) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"Error in Lapack xTREXC %d",info);
+      SlepcCheckLapackInfo("trexc",info);
       /* recover original eigenvalues from T matrix */
       for (j=i;j<n;j++) {
         wr[j] = T[j+j*ld];
@@ -506,8 +420,6 @@ static PetscErrorCode DSSort_NHEP_Total(DS ds,PetscScalar *wr,PetscScalar *wi)
 #endif
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSSort_NHEP"
 PetscErrorCode DSSort_NHEP(DS ds,PetscScalar *wr,PetscScalar *wi,PetscScalar *rr,PetscScalar *ri,PetscInt *k)
 {
   PetscErrorCode ierr;
@@ -521,8 +433,6 @@ PetscErrorCode DSSort_NHEP(DS ds,PetscScalar *wr,PetscScalar *wi,PetscScalar *rr
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSUpdateExtraRow_NHEP"
 PetscErrorCode DSUpdateExtraRow_NHEP(DS ds)
 {
   PetscErrorCode ierr;
@@ -545,8 +455,59 @@ PetscErrorCode DSUpdateExtraRow_NHEP(DS ds)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSSolve_NHEP"
+/*
+   Reduce a matrix A to upper Hessenberg form Q'*A*Q, where Q is an orthogonal matrix.
+   The result overwrites A. Matrix A has the form
+
+         [ S | * ]
+     A = [-------]
+         [ R | H ]
+
+  where S is an upper (quasi-)triangular matrix of order k, H is an upper Hessenberg
+  matrix of order n-k, and R is all zeros except the first row (the arrow).
+  The algorithm uses elementary reflectors to annihilate entries in the arrow, and
+  then proceeds upwards. 
+  If ilo>1, then it is assumed that the first ilo-1 entries of the arrow are zero, and
+  hence the first ilo-1 rows and columns of Q are set to the identity matrix.
+
+  Required workspace is 2*n.
+*/
+static PetscErrorCode ArrowHessenberg(PetscBLASInt n,PetscBLASInt k,PetscBLASInt ilo,PetscScalar *A,PetscBLASInt lda,PetscScalar *Q,PetscBLASInt ldq,PetscScalar *work)
+{
+#if defined(SLEPC_MISSING_LAPACK_LARFG) || defined(SLEPC_MISSING_LAPACK_LARF)
+  PetscFunctionBegin;
+  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"LARFG/LARF - Lapack routines are unavailable");
+#else
+  PetscBLASInt i,j,n0,m,inc=1,incn=-1;
+  PetscScalar  t,*v=work,*w=work+n,tau,tauc;
+
+  PetscFunctionBegin;
+  m = n-ilo+1;
+  for (i=k;i>ilo;i--) {
+    for (j=0;j<=i-ilo;j++) v[j] = A[i+(i-j-1)*lda];  /* _larfg does not allow negative inc, so use buffer */
+    n0 = i-ilo+1;
+    PetscStackCallBLAS("LAPACKlarfg",LAPACKlarfg_(&n0,v,v+1,&inc,&tau));
+    for (j=1;j<=i-ilo;j++) v[j] = PetscConj(v[j]);
+    tauc = PetscConj(tau);
+    A[i+(i-1)*lda] = v[0];
+    v[0] = 1.0;
+    PetscStackCallBLAS("LAPACKlarf",LAPACKlarf_("R",&i,&n0,v,&incn,&tauc,A+(ilo-1)*lda,&lda,w));
+    for (j=1;j<=i-ilo;j++) A[i+(i-j-1)*lda] = 0.0;
+    PetscStackCallBLAS("LAPACKlarf",LAPACKlarf_("L",&n0,&m,v,&incn,&tau,A+ilo-1+(ilo-1)*lda,&lda,w));
+    PetscStackCallBLAS("LAPACKlarf",LAPACKlarf_("L",&n0,&m,v,&incn,&tau,Q+ilo-1+(ilo-1)*ldq,&ldq,w));
+  }
+  /* trivial in-place transposition of Q */
+  for (j=ilo-1;j<k;j++) {
+    for (i=j;i<k;i++) {
+      t = Q[i+j*ldq];
+      if (i!=j) Q[i+j*ldq] = PetscConj(Q[j+i*ldq]);
+      Q[j+i*ldq] = PetscConj(t);
+    }
+  }
+  PetscFunctionReturn(0);
+#endif
+}
+
 PetscErrorCode DSSolve_NHEP(DS ds,PetscScalar *wr,PetscScalar *wi)
 {
 #if defined(SLEPC_MISSING_LAPACK_GEHRD) || defined(SLEPC_MISSING_LAPACK_ORGHR) || defined(PETSC_MISSING_LAPACK_HSEQR)
@@ -556,7 +517,7 @@ PetscErrorCode DSSolve_NHEP(DS ds,PetscScalar *wr,PetscScalar *wi)
   PetscErrorCode ierr;
   PetscScalar    *work,*tau;
   PetscInt       i,j;
-  PetscBLASInt   ilo,lwork,info,n,ld;
+  PetscBLASInt   ilo,lwork,info,n,k,ld;
   PetscScalar    *A = ds->mat[DS_MAT_A];
   PetscScalar    *Q = ds->mat[DS_MAT_Q];
 
@@ -567,15 +528,15 @@ PetscErrorCode DSSolve_NHEP(DS ds,PetscScalar *wr,PetscScalar *wi)
   ierr = PetscBLASIntCast(ds->n,&n);CHKERRQ(ierr);
   ierr = PetscBLASIntCast(ds->ld,&ld);CHKERRQ(ierr);
   ierr = PetscBLASIntCast(ds->l+1,&ilo);CHKERRQ(ierr);
-  ierr = DSAllocateWork_Private(ds,ld+ld*ld,0,0);CHKERRQ(ierr);
+  ierr = PetscBLASIntCast(ds->k,&k);CHKERRQ(ierr);
+  ierr = DSAllocateWork_Private(ds,ld+6*ld,0,0);CHKERRQ(ierr);
   tau  = ds->work;
   work = ds->work+ld;
-  lwork = ld*ld;
+  lwork = 6*ld;
 
   /* initialize orthogonal matrix */
   ierr = PetscMemzero(Q,ld*ld*sizeof(PetscScalar));CHKERRQ(ierr);
-  for (i=0;i<n;i++)
-    Q[i+i*ld] = 1.0;
+  for (i=0;i<n;i++) Q[i+i*ld] = 1.0;
   if (n==1) { /* quick return */
     wr[0] = A[0];
     wi[0] = 0.0;
@@ -584,16 +545,20 @@ PetscErrorCode DSSolve_NHEP(DS ds,PetscScalar *wr,PetscScalar *wi)
 
   /* reduce to upper Hessenberg form */
   if (ds->state<DS_STATE_INTERMEDIATE) {
-    PetscStackCallBLAS("LAPACKgehrd",LAPACKgehrd_(&n,&ilo,&n,A,&ld,tau,work,&lwork,&info));
-    if (info) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"Error in Lapack xGEHRD %d",info);
-    for (j=0;j<n-1;j++) {
-      for (i=j+2;i<n;i++) {
-        Q[i+j*ld] = A[i+j*ld];
-        A[i+j*ld] = 0.0;
+    if (PETSC_FALSE && k>0) {
+      ierr = ArrowHessenberg(n,k,ilo,A,ld,Q,ld,work);CHKERRQ(ierr);
+    } else {
+      PetscStackCallBLAS("LAPACKgehrd",LAPACKgehrd_(&n,&ilo,&n,A,&ld,tau,work,&lwork,&info));
+      SlepcCheckLapackInfo("gehrd",info);
+      for (j=0;j<n-1;j++) {
+        for (i=j+2;i<n;i++) {
+          Q[i+j*ld] = A[i+j*ld];
+          A[i+j*ld] = 0.0;
+        }
       }
+      PetscStackCallBLAS("LAPACKorghr",LAPACKorghr_(&n,&ilo,&n,Q,&ld,tau,work,&lwork,&info));
+      SlepcCheckLapackInfo("orghr",info);
     }
-    PetscStackCallBLAS("LAPACKorghr",LAPACKorghr_(&n,&ilo,&n,Q,&ld,tau,work,&lwork,&info));
-    if (info) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"Error in Lapack xORGHR %d",info);
   }
 
   /* compute the (real) Schur form */
@@ -618,17 +583,59 @@ PetscErrorCode DSSolve_NHEP(DS ds,PetscScalar *wr,PetscScalar *wi)
   PetscStackCallBLAS("LAPACKhseqr",LAPACKhseqr_("S","V",&n,&ilo,&n,A,&ld,wr,Q,&ld,work,&lwork,&info));
   if (wi) for (i=ds->l;i<n;i++) wi[i] = 0.0;
 #endif
-  if (info) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"Error in Lapack xHSEQR %d",info);
+  SlepcCheckLapackInfo("hseqr",info);
   PetscFunctionReturn(0);
 #endif
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSTruncate_NHEP"
+PetscErrorCode DSSynchronize_NHEP(DS ds,PetscScalar eigr[],PetscScalar eigi[])
+{
+  PetscErrorCode ierr;
+  PetscInt       ld=ds->ld,l=ds->l,k;
+  PetscMPIInt    n,rank,off=0,size,ldn;
+
+  PetscFunctionBegin;
+  k = (ds->n-l)*ld;
+  if (ds->state>DS_STATE_RAW) k += (ds->n-l)*ld;
+  if (eigr) k += ds->n-l; 
+  if (eigi) k += ds->n-l; 
+  ierr = DSAllocateWork_Private(ds,k,0,0);CHKERRQ(ierr);
+  ierr = PetscMPIIntCast(k*sizeof(PetscScalar),&size);CHKERRQ(ierr);
+  ierr = PetscMPIIntCast(ds->n-l,&n);CHKERRQ(ierr);
+  ierr = PetscMPIIntCast(ld*(ds->n-l),&ldn);CHKERRQ(ierr);
+  ierr = MPI_Comm_rank(PetscObjectComm((PetscObject)ds),&rank);CHKERRQ(ierr);
+  if (!rank) {
+    ierr = MPI_Pack(ds->mat[DS_MAT_A]+l*ld,ldn,MPIU_SCALAR,ds->work,size,&off,PetscObjectComm((PetscObject)ds));CHKERRQ(ierr);
+    if (ds->state>DS_STATE_RAW) {
+      ierr = MPI_Pack(ds->mat[DS_MAT_Q]+l*ld,ldn,MPIU_SCALAR,ds->work,size,&off,PetscObjectComm((PetscObject)ds));CHKERRQ(ierr);
+    }
+    if (eigr) {
+      ierr = MPI_Pack(eigr+l,n,MPIU_SCALAR,ds->work,size,&off,PetscObjectComm((PetscObject)ds));CHKERRQ(ierr);
+    }
+    if (eigi) {
+      ierr = MPI_Pack(eigi+l,n,MPIU_SCALAR,ds->work,size,&off,PetscObjectComm((PetscObject)ds));CHKERRQ(ierr);
+    }
+  }
+  ierr = MPI_Bcast(ds->work,size,MPI_BYTE,0,PetscObjectComm((PetscObject)ds));CHKERRQ(ierr);
+  if (rank) {
+    ierr = MPI_Unpack(ds->work,size,&off,ds->mat[DS_MAT_A]+l*ld,ldn,MPIU_SCALAR,PetscObjectComm((PetscObject)ds));CHKERRQ(ierr);
+    if (ds->state>DS_STATE_RAW) {
+      ierr = MPI_Unpack(ds->work,size,&off,ds->mat[DS_MAT_Q]+l*ld,ldn,MPIU_SCALAR,PetscObjectComm((PetscObject)ds));CHKERRQ(ierr);
+    }
+    if (eigr) {
+      ierr = MPI_Unpack(ds->work,size,&off,eigr+l,n,MPIU_SCALAR,PetscObjectComm((PetscObject)ds));CHKERRQ(ierr);
+    }
+    if (eigi) {
+      ierr = MPI_Unpack(ds->work,size,&off,eigi+l,n,MPIU_SCALAR,PetscObjectComm((PetscObject)ds));CHKERRQ(ierr);
+    }
+  }
+  PetscFunctionReturn(0);
+}
+
 PetscErrorCode DSTruncate_NHEP(DS ds,PetscInt n)
 {
-  PetscInt       i,newn,ld=ds->ld,l=ds->l;
-  PetscScalar    *A;
+  PetscInt    i,newn,ld=ds->ld,l=ds->l;
+  PetscScalar *A;
 
   PetscFunctionBegin;
   if (ds->state==DS_STATE_CONDENSED) ds->t = ds->n;
@@ -649,8 +656,6 @@ PetscErrorCode DSTruncate_NHEP(DS ds,PetscInt n)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSCond_NHEP"
 PetscErrorCode DSCond_NHEP(DS ds,PetscReal *cond)
 {
 #if defined(PETSC_MISSING_LAPACK_GETRF) || defined(PETSC_MISSING_LAPACK_GETRI) || defined(SLEPC_MISSING_LAPACK_LANGE) || defined(SLEPC_MISSING_LAPACK_LANHS)
@@ -685,9 +690,9 @@ PetscErrorCode DSCond_NHEP(DS ds,PetscReal *cond)
 
   /* norm of inv(A) */
   PetscStackCallBLAS("LAPACKgetrf",LAPACKgetrf_(&n,&n,A,&ld,ipiv,&info));
-  if (info) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"Error in Lapack xGETRF %d",info);
+  SlepcCheckLapackInfo("getrf",info);
   PetscStackCallBLAS("LAPACKgetri",LAPACKgetri_(&n,A,&ld,ipiv,work,&lwork,&info));
-  if (info) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"Error in Lapack xGETRI %d",info);
+  SlepcCheckLapackInfo("getri",info);
   hin = LAPACKlange_("I",&n,&n,A,&ld,rwork);
 
   *cond = hn*hin;
@@ -695,8 +700,6 @@ PetscErrorCode DSCond_NHEP(DS ds,PetscReal *cond)
 #endif
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSTranslateHarmonic_NHEP"
 PetscErrorCode DSTranslateHarmonic_NHEP(DS ds,PetscScalar tau,PetscReal beta,PetscBool recover,PetscScalar *gin,PetscReal *gamma)
 {
 #if defined(PETSC_MISSING_LAPACK_GETRF) || defined(PETSC_MISSING_LAPACK_GETRS)
@@ -733,19 +736,16 @@ PetscErrorCode DSTranslateHarmonic_NHEP(DS ds,PetscScalar tau,PetscReal beta,Pet
     g[n-1] = beta;
 
     /* g = (A-tau*eye(n))'\b */
-    for (i=0;i<n;i++)
-      B[i+i*ld] -= tau;
+    for (i=0;i<n;i++) B[i+i*ld] -= tau;
     PetscStackCallBLAS("LAPACKgetrf",LAPACKgetrf_(&n,&n,B,&ld,ipiv,&info));
-    if (info<0) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_LIB,"Bad argument to LU factorization");
-    if (info>0) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_MAT_LU_ZRPVT,"Bad LU factorization");
+    SlepcCheckLapackInfo("getrf",info);
     ierr = PetscLogFlops(2.0*n*n*n/3.0);CHKERRQ(ierr);
     PetscStackCallBLAS("LAPACKgetrs",LAPACKgetrs_("C",&n,&one,B,&ld,ipiv,g,&ld,&info));
-    if (info) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_LIB,"GETRS - Bad solve");
+    SlepcCheckLapackInfo("getrs",info);
     ierr = PetscLogFlops(2.0*n*n-n);CHKERRQ(ierr);
 
     /* A = A + g*b' */
-    for (i=0;i<n;i++)
-      A[i+(n-1)*ld] += g[i]*beta;
+    for (i=0;i<n;i++) A[i+(n-1)*ld] += g[i]*beta;
 
   } else { /* recover */
 
@@ -770,16 +770,13 @@ PetscErrorCode DSTranslateHarmonic_NHEP(DS ds,PetscScalar tau,PetscReal beta,Pet
   /* Compute gamma factor */
   if (gamma) {
     gnorm = 0.0;
-    for (i=0;i<n;i++)
-      gnorm = gnorm + PetscRealPart(g[i]*PetscConj(g[i]));
+    for (i=0;i<n;i++) gnorm = gnorm + PetscRealPart(g[i]*PetscConj(g[i]));
     *gamma = PetscSqrtReal(1.0+gnorm);
   }
   PetscFunctionReturn(0);
 #endif
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DSCreate_NHEP"
 PETSC_EXTERN PetscErrorCode DSCreate_NHEP(DS ds)
 {
   PetscFunctionBegin;
@@ -788,11 +785,11 @@ PETSC_EXTERN PetscErrorCode DSCreate_NHEP(DS ds)
   ds->ops->vectors       = DSVectors_NHEP;
   ds->ops->solve[0]      = DSSolve_NHEP;
   ds->ops->sort          = DSSort_NHEP;
+  ds->ops->synchronize   = DSSynchronize_NHEP;
   ds->ops->truncate      = DSTruncate_NHEP;
   ds->ops->update        = DSUpdateExtraRow_NHEP;
   ds->ops->cond          = DSCond_NHEP;
   ds->ops->transharm     = DSTranslateHarmonic_NHEP;
-  ds->ops->normalize     = DSNormalize_NHEP;
   PetscFunctionReturn(0);
 }
 
