@@ -265,14 +265,16 @@ PetscErrorCode STMatSetHermitian(ST st,Mat M)
 
   PetscFunctionBegin;
 #if defined(PETSC_USE_COMPLEX)
-  mherm = PETSC_FALSE;
-  for (i=0;i<st->nmat;i++) {
-    ierr = MatIsHermitianKnown(st->A[i],&set,&aherm);CHKERRQ(ierr);
-    if (!set) aherm = PETSC_FALSE;
-    mherm = (mherm && aherm)? PETSC_TRUE: PETSC_FALSE;
-    if (PetscRealPart(st->sigma)==0.0) break;
+  if (PetscImaginaryPart(st->sigma)!=0.0) mherm = PETSC_FALSE;
+  else {
+    mherm = PETSC_TRUE;
+    for (i=0;i<st->nmat;i++) {
+      ierr = MatIsHermitianKnown(st->A[i],&set,&aherm);CHKERRQ(ierr);
+      if (!set) aherm = PETSC_FALSE;
+      if (!aherm) { mherm = PETSC_FALSE; break; }
+      if (PetscRealPart(st->sigma)==0.0) break;
+    }
   }
-  mherm = (mherm && PetscImaginaryPart(st->sigma)==0.0)? PETSC_TRUE: PETSC_FALSE;
   ierr = MatSetOption(M,MAT_HERMITIAN,mherm);CHKERRQ(ierr);
 #endif
   PetscFunctionReturn(0);
